/*
 * CommandLine.cs
 *
 * File Statistics
 * Version: 1.0
 *
 * Copyright (c) 2005, TopCoder, Inc. All rights reserved
 *
 * Authors: WishingBone, TCSDEVELOPER
 */

using System;
using System.IO;
using TopCoder.Util.ConfigurationManager;
using TopCoder.Util.CommandLine;

namespace TopCoder.File.Statistics
{
    /// <summary>
    /// The command line entry point of the File Statistics utility.
    /// </summary>
    ///
    /// <remarks>
    /// <para>
    /// It uses <see cref="ExtendedFileStatistics"/> to fulfill the task.
    /// When argument is invalid, help message should be printed.
    /// </para>
    /// <para>
    /// The command line syntax:
    /// <code>
    /// CommandLine.exe [-c:config-file] [-n:namespace] [-b:base-path] [-r] [-o:output-file] path [path2 path3 ...]
    /// where:
    /// * -c:config-file: provides a config file to initialize from
    /// * -n:namespace: provides a namespace to initialize from
    /// *    If neither is provided the default namespace is used.
    /// *    Provided namespace should be preloaded.
    /// * -b:base-path: all path will be relative to it.
    /// * -r: whether to recursively traverse sub-directory.
    /// * -o:output-file: the output filename. If missing, output is printed to console.
    /// </code>
    /// </para>
    /// </remarks>
    ///
    /// <threadsafety static="true" instance="true"/>
    public class CommandLine
    {
        /// <summary>
        /// The namespace to used if no namespace is provided.
        /// </summary>
        public static readonly string Namespace = "TopCoder.File.Statistics.Main";

        /// <summary>
        /// The command line parser.
        /// Initialized in InitParser().
        /// </summary>
        private static CommandLineParser parser;

        /// <summary>
        /// Prevents this class from being instantiated.
        /// </summary>
        private CommandLine()
        {
        }

        /// <summary>
        /// The command line entry.
        /// </summary>
        ///
        /// <param name="args">
        /// The command line arguments. Please refer to the class documentation.
        /// </param>
        public static void Main(string[] args)
        {
            try
            {
                // init parser
                InitParser();

                // parse command line
                parser.Parse(args);

                // at least 1 argument
                if (parser.Arguments.Length < 1)
                {
                    PrintHelp();
                    return;
                }

                // load configuration
                LoadConfiguration();

                // determine namespace and instantiate ExtendedStatistics
                string nameSpace = ((parser.InputtedSwitches.Contains("n")) ?
                    GetSwitchValue("n") : Namespace);

                ExtendedFileStatistics statistics = new ExtendedFileStatistics(nameSpace);

                // set the base path
                if (parser.InputtedSwitches.Contains("b"))
                {
                    statistics.Reporting.Reset(new DirectoryInfo(GetSwitchValue("b")));
                }

                // do we want recurse
                bool recurse = parser.InputtedSwitches.Contains("r");

                // process all arguments
                foreach (string files in parser.Arguments)
                {
                    // assume it's a directory
                    DirectoryInfo dir = new DirectoryInfo(files);

                    // use ProcessDirectory or ProcessFile accordingly
                    if ((dir.Attributes & FileAttributes.Directory) != 0)
                    {
                        statistics.ProcessDirectory(dir, recurse);
                    }
                    else
                    {
                        statistics.ProcessFile(new FileInfo(files));
                    }
                }

                // print report
                if (parser.InputtedSwitches.Contains("o"))
                {
                    statistics.PrintReport(new FileInfo(GetSwitchValue("o")));
                }
                else
                {
                    statistics.PrintReport();
                }
            }
            catch (Exception e)
            {
                // print the error, then print the help
                Console.WriteLine(string.Format("Error: {0}{1}{2}",
                    e.Message, Environment.NewLine, e.StackTrace));
                Console.WriteLine();
                PrintHelp();
            }
        }

        /// <summary>
        /// Prints help messages to console.
        /// </summary>
        private static void PrintHelp()
        {
            Console.WriteLine("The command line syntax:");
            Console.WriteLine("CommandLine.exe [-c:config-file] [-n:namespace] [-b:base-path] [-r] "
                + "[-o:output-file] path [path2 path3 ...]");
            Console.WriteLine("where:");
            Console.WriteLine("* -c:config-file: provides a config file to initialize from");
            Console.WriteLine("* -n:namespace: provides a namespace to initialize from");
            Console.WriteLine("*    If neither is provided the default namespace is used.");
            Console.WriteLine("*    Provided namespace should be preloaded.");
            Console.WriteLine("* -b:base-path: all path will be relative to it.");
            Console.WriteLine("* -r: whether to recursively traverse sub-directory.");
            Console.WriteLine("* -o:output-file: the output filename. If missing, output is printed to console.");
        }

        /// <summary>
        /// Initializes the command line parser.
        /// </summary>
        private static void InitParser()
        {
            parser = new CommandLineParser();
            // -c
            parser.AddSwitch(new CommandLineSwitch(
                "c", "config-file", "Provides a config file to initialize from"));
            // -n
            parser.AddSwitch(new CommandLineSwitch(
                "n", "namespace", "Provides a namespace to initialize from"));
            // -b
            parser.AddSwitch(new CommandLineSwitch(
                "b", "base-path", "All path will be relative to it"));
            // -r
            parser.AddSwitch(new CommandLineSwitch(
                "r", "recurse", "Whether to recursively traverse sub-directory"));
            // -o
            parser.AddSwitch(new CommandLineSwitch(
                "o", "output-file", "The output filename. If missing, output is printed to console."));
        }

        /// <summary>
        /// Loads configuration as specified by the command line argument.
        /// </summary>
        private static void LoadConfiguration()
        {
            ConfigManager cm = ConfigManager.GetInstance();
            if (parser.InputtedSwitches.Contains("c"))
            {
                // is namespace also specified?
                if (parser.InputtedSwitches.Contains("n"))
                {
                    // load config into the specified namespace
                    cm.LoadFile(GetSwitchValue("c"), GetSwitchValue("n"));
                }
                else
                {
                    // just load it
                    cm.LoadFile(GetSwitchValue("c"));
                }
            }
        }

        /// <summary>
        /// Get switch value as a string
        /// </summary>
        ///
        /// <param name="sw">
        /// The switch.
        /// </param>
        ///
        /// <returns>
        /// The switch value as a string, or null if the switch isn't supplied.
        /// </returns>
        private static string GetSwitchValue(string sw)
        {
            if (!(parser.InputtedSwitches.Contains(sw)))
            {
                // switch isn't supplied
                return null;
            }

            // get the switch value
            return (string) ((CommandLineSwitch) parser.InputtedSwitches[sw]).Result[0];
        }

    }
}
