/*
 * Helper.cs
 *
 * File Statistics
 * Version: 1.0
 *
 * Copyright (c) 2005, TopCoder, Inc. All rights reserved
 *
 * Author: TCSDEVELOPER
 */

using System;
using System.Collections;
using System.Reflection;
using TopCoder.Util.ConfigurationManager;

namespace TopCoder.File.Statistics
{
    /// <summary>
    /// Common functions containing various static helper methods.
    /// </summary>
    ///
    /// <threadsafety static="true" instance="true"/>
    internal sealed class Helper
    {
        /// <summary>
        /// Adds list contents to the dictionary as its key.
        /// </summary>
        ///
        /// <remarks>
        /// The dictionary value will be null.
        /// </remarks>
        ///
        /// <param name="list">
        /// The list to copy.
        /// </param>
        /// <param name="dict">
        /// The dictionary to add.
        /// </param>
        ///
        /// <exception cref="ArgumentException">
        /// Duplicate keys exist in the list.
        /// </exception>
        internal static void AddListToDictionary(IList list, IDictionary dict)
        {
            try
            {
                // iterate and add it
                foreach (object o in list)
                {
                    // this may throw AE if the key already exists
                    dict.Add(o, null);
                }
            }
            catch (ArgumentException)
            {
                // just re-throw
                throw;
            }
            catch (Exception e)
            {
                // encapsulate all others in ArgumentException
                throw new ArgumentException("List should not contain duplicate value", e);
            }
        }

        /// <summary>
        /// Dynamically creates an instance of a class
        /// whose constructor takes 2 string parameters (the namespace and prefix).
        /// </summary>
        ///
        /// <param name="cm">
        /// The config manager instance to load assembly name and class name from.
        /// </param>
        /// <param name="nameSpace">
        /// The namespace to load configuration values from.
        /// </param>
        /// <param name="assemblyProperty">
        /// The property name for the assembly to load the class from.
        /// If null, it'll load the class from the currently running assembly.
        /// </param>
        /// <param name="classProperty">
        /// The property name for the class name to instantiate.
        /// </param>
        /// <param name="args">
        /// The parameters to be passed to the constructor.
        /// </param>
        /// <param name="baseClass">
        /// The base class from which classProperty must derive or implement.
        /// </param>
        ///
        /// <returns>
        /// A reference to the newly created object.
        /// </returns>
        ///
        /// <exception cref="ConfigurationException">
        /// Fail to create the object instance.
        /// </exception>
        internal static object CreateInstance(ConfigManager cm, string nameSpace,
            string assemblyProperty, string classProperty, string[] args, Type baseClass)
        {
            try
            {
                // CM 2.0's way:
                // return cm.CreateInstance(nameSpace, classProperty, assemblyProperty,
                //    args, baseClass);
                // but classProperty must contain fully qualified class name + assembly name.
                // Without assembly name, CM tries to load it
                // from TopCoder.Util.ConfigurationManager only (a bug?)
                // instead of the calling assembly

                // thus, we create our own implementation
                // which works without assembly name and
                // only class name needs to be specified in the config file

                // get the class name
                string className = cm.GetValue(nameSpace, classProperty);

                // use current assembly if null
                string assembly;
                if (assemblyProperty == null)
                {
                    assembly = Assembly.GetExecutingAssembly().CodeBase;
                }
                else
                {
                    assembly = cm.GetValue(nameSpace, assemblyProperty);
                }

                // instantiate it
                AppDomain currentDomain = AppDomain.CurrentDomain;
                object instance = currentDomain.CreateInstanceFromAndUnwrap(
                    assembly,
                    className,
                    false,
                    BindingFlags.Default,
                    null,
                    args,
                    null,
                    null,
                    null
                    );

                // check type
                if (!(baseClass.IsAssignableFrom(instance.GetType())))
                {
                    throw new ConfigurationException(string.Format(
                        "{0} is not subclass of {1}", className, baseClass.Name));
                }

                return instance;
            }
            catch(ConfigurationException)
            {
                // just re-throw
                throw;
            }
            catch(Exception e)
            {
                throw new ConfigurationException(string.Format(
                    "Error creating instance using assembly property {0} and class property {1}",
                    assemblyProperty == null ? "" : assemblyProperty, classProperty), e);
            }
        }
    }
}
