/*
 * IFileMatcher.cs
 *
 * File Statistics
 * Version: 1.0
 *
 * Copyright (c) 2005, TopCoder, Inc. All rights reserved
 *
 * Authors: WishingBone, TCSDEVELOPER
 */

using System;
using System.IO;
using TopCoder.File.Statistics.Matcher;

namespace TopCoder.File.Statistics
{
    /// <summary>
    /// Defines the contract to match a certain file
    /// (in order for an attaching <see cref="IFileProcessor"/> to process the file).
    /// </summary>
    ///
    /// <remarks>
    /// <para>
    /// An implementation of this interface is used by <see cref="IFileProcessor"/>
    /// to check whether a particular file matches a criteria and can be processed
    /// by the corresponding <see cref="IFileProcessor"/>.
    /// </para>
    /// <para>
    /// Currently, this component ships with seven implementations which try to match
    /// a file based on various criteria:
    /// <list type="table">
    /// <listheader>
    ///     <term>Implementation</term>
    ///     <description>Match criteria</description>
    /// </listheader>
    /// <item>
    ///     <term><see cref="AnyFileMatcher"/></term>
    ///     <description>Any file</description>
    /// </item>
    /// <item>
    ///     <term><see cref="ExtensionMatcher"/></term>
    ///     <description>File name extension</description>
    /// </item>
    /// <item>
    ///     <term><see cref="RegexMatcher"/></term>
    ///     <description>File name regular expression matching</description>
    /// </item>
    /// <item>
    ///     <term><see cref="TextFileMatcher"/></term>
    ///     <description>Text file content</description>
    /// </item>
    /// <item>
    ///     <term><see cref="BinaryFileMatcher"/></term>
    ///     <description>Binary file content</description>
    /// </item>
    /// <item>
    ///     <term><see cref="TypeNameMatcher"/></term>
    ///     <description>File type/content</description>
    /// </item>
    /// <item>
    ///     <term><see cref="MimeMatcher"/></term>
    ///     <description>File MIME signature/content</description>
    /// </item>
    /// </list>
    /// </para>
    /// <para>
    /// <b>Constructor contract:</b>
    /// All implementation should have a constructor with two string arguments:
    /// nameSpace and prefix (<c>FileMatcher(string nameSpace, string prefix)</c>)
    /// in order to be instantiated via reflection from a configuration file.
    /// This requirement can be dropped if it's not necessary to use it this way.
    /// Implementation-specific configuration can be written in the configuration
    /// file with the specified prefix as its property name.
    /// </para>
    /// <para>
    /// <b>Thread safety:</b>
    /// Implementations are advised to be stateless and thread-safe. But it's not a requirement.
    /// </para>
    /// </remarks>
    public interface IFileMatcher
    {
        /// <summary>
        /// Determines if the file can/will be processed by the processor
        /// that this matcher is associated with.
        /// </summary>
        ///
        /// <param name="file">
        /// The file to match.
        /// </param>
        ///
        /// <returns>
        /// <b>true</b> if the file matches the criteria; otherwise, <b>false</b>.
        /// </returns>
        ///
        /// <exception cref="ArgumentNullException">
        /// <paramref name="file"/> is a null reference.
        /// </exception>
        /// <exception cref="IOException">
        /// An IO operation fails.
        /// </exception>
        /// <exception cref="FileMatchingException">
        /// An error, other than IO error, occurs.
        /// </exception>
        bool Match(FileInfo file);
    }
}
