/*
 * IFileProcessor.cs
 *
 * File Statistics
 * Version: 1.0
 *
 * Copyright (c) 2005, TopCoder, Inc. All rights reserved
 *
 * Authors: WishingBone, TCSDEVELOPER
 */

using System;
using System.IO;
using System.Collections;
using TopCoder.File.Statistics.Processor.LineCounter;

namespace TopCoder.File.Statistics
{
    /// <summary>
    /// Defines the contract to process a file and obtains its <see cref="StatsResult"/>.
    /// </summary>
    ///
    /// <remarks>
    /// <para>
    /// One file processor defines an algorithm to produce statistical
    /// result for a certain class of files against certain metrics
    /// (for instance, count line numbers for a text file).
    /// Multiple <see cref="IFileProcessor"/> are aggregated into <see cref="FileStatistics"/>,
    /// which can run in two modes (process with first matching processor
    /// and process with each matching processor).
    /// File processors are mapped with an alias. This alias is
    /// dynamically provided by the user and
    /// the reporting can be grouped against this alias.
    /// </para>
    /// <para>
    /// Currently, this component ships with two implementations:
    /// <list type="table">
    /// <listheader>
    ///     <term>Implementation</term>
    ///     <description>Processing</description>
    /// </listheader>
    /// <item>
    ///     <term><see cref="TextLineCounter"/></term>
    ///     <description>Counts lines for a text file</description>
    /// </item>
    /// <item>
    ///     <term><see cref="CStyleLineCounter"/></term>
    ///     <description>A simple implementation to count C style source file.</description>
    /// </item>
    /// </list>
    /// </para>
    /// <para>
    /// <b>Constructor contract:</b>
    /// All implementation should have a constructor with two string arguments:
    /// nameSpace and prefix (<c>FileProcessor(string nameSpace, string prefix)</c>)
    /// in order to be instantiated via reflection from a configuration file.
    /// This requirement can be dropped if it's not necessary to use it this way.
    /// Implementation-specific configuration can be written in the configuration
    /// file with the specified prefix as its property name.
    /// </para>
    /// <para>
    /// <b>Thread safety:</b>
    /// Implementations are advised to be stateless and thread-safe. But it's not a requirement.
    /// </para>
    /// </remarks>
    public interface IFileProcessor
    {
        /// <summary>
        /// Gets the alias name of the processor.
        /// </summary>
        ///
        /// <remarks>
        /// An alias is the official name of the processor
        /// which is also used to classify the statistic result against.
        /// This should be non-empty.
        /// </remarks>
        ///
        /// <value>
        /// The alias name of the processor.
        /// </value>
        string Alias
        {
            get;
        }

        /// <summary>
        /// Gets the file matcher associated with the processor
        /// in order to determine whether a file can/should be processed with
        /// the processor.
        /// </summary>
        ///
        /// <value>
        /// The <see cref="IFileMatcher"/> instance associated with this processor
        /// which determines whether a file can/should be processed with
        /// the processor.
        /// </value>
        IFileMatcher FileMatcher
        {
            get;
        }

        /// <summary>
        /// Gets a list of all the supported metrics by this processor.
        /// </summary>
        ///
        /// <remarks>
        /// This method is mostly for end user. It does not fulfill any
        /// responsibility in the current framework.
        /// </remarks>
        ///
        /// <value>
        /// A list of <see cref="Metric"/> supported by this processor.
        /// </value>
        IList SupportedMetrics
        {
            get;
        }

        /// <summary>
        /// Processes the file and generates the statistic result.
        /// </summary>
        ///
        /// <remarks>
        /// This will actually process the file and gather the file statistic.
        /// The result is then returned via <see cref="StatsResult"/>.
        /// A new result should be generated each time,
        /// whose keys are the metrics returned from <see cref="SupportedMetrics"/>.
        /// </remarks>
        ///
        /// <param name="file">
        /// The file to process.
        /// </param>
        ///
        /// <returns>
        /// A <see cref="StatsResult"/> representing the statistic result for the file.
        /// </returns>
        ///
        /// <exception cref="ArgumentNullException">
        /// <paramref name="file"/> is a null reference.
        /// </exception>
        /// <exception cref="IOException">
        /// Any IO operation fails.
        /// </exception>
        /// <exception cref="FileProcessingException">
        /// The processor fails.
        /// </exception>
        StatsResult Process(FileInfo file);
    }
}
