/*
 * IStatsReporting.cs
 *
 * File Statistics
 * Version: 1.0
 *
 * Copyright (c) 2005, TopCoder, Inc. All rights reserved
 *
 * Authors: WishingBone, TCSDEVELOPER
 */

using System;
using System.IO;
using TopCoder.File.Statistics.Reporting;

namespace TopCoder.File.Statistics
{
    /// <summary>
    /// Defines the contract to aggregate and render the statistical report.
    /// </summary>
    ///
    /// <remarks>
    /// <para>
    /// Result (<see cref="StatsResult"/> is fed into this interface one by one and
    /// the file report can then be generated.
    /// There is a way to reset (clear) the current aggregated results.
    /// </para>
    /// <para>
    /// Currently, this component ships with two implementations:
    /// <list type="table">
    /// <listheader>
    ///     <term>Implementation</term>
    ///     <description>Reporting</description>
    /// </listheader>
    /// <item>
    ///     <term><see cref="BasicStatsReporting"/></term>
    ///     <description>Standard reporting to standard output.</description>
    /// </item>
    /// <item>
    ///     <term><see cref="XmlStatsReporting"/></term>
    ///     <description>Reporting to xml format.</description>
    /// </item>
    /// </list>
    /// </para>
    /// <para>
    /// <b>Constructor contract:</b>
    /// All implementation should have a constructor with two string arguments:
    /// nameSpace and prefix (<c>StatsReporting(string nameSpace, string prefix)</c>)
    /// in order to be instantiated via reflection from a configuration file.
    /// This requirement can be dropped if it's not necessary to use it this way.
    /// Implementation-specific configuration can be written in the configuration
    /// file with the specified prefix as its property name.
    /// </para>
    /// <para>
    /// <b>Thread safety:</b>
    /// There is no thread-safety requirement/recommendation for the implementations.
    /// </para>
    /// </remarks>
    public interface IStatsReporting
    {
        /// <summary>
        /// Aggregates the result into the reporting implementation.
        /// </summary>
        ///
        /// <remarks>
        /// The results are kept by the reporting itself.
        /// </remarks>
        ///
        /// <param name="processor">
        /// The processor that generated the result.
        /// </param>
        /// <param name="file">
        /// The file that is processed.
        /// </param>
        /// <param name="result">
        /// The processing result
        /// </param>
        ///
        /// <exception cref="ArgumentNullException">
        /// Any of the argument is a null reference.
        /// </exception>
        /// <exception cref="StatsReportingException">
        /// <paramref name="file"/> is not in base dir.
        /// </exception>
        void Aggregate(IFileProcessor processor, FileInfo file, StatsResult result);

        /// <summary>
        /// Generates the report with all the aggregated results.
        /// </summary>
        ///
        /// <returns>
        /// The generated report.
        /// </returns>
        ///
        /// <exception cref="StatsReportingException">
        /// An error occurs.
        /// </exception>
        string GenerateReport();

        /// <summary>
        /// Clears all the aggregated result and sets the base dir (which is nullable).
        /// </summary>
        ///
        /// <param name="baseDirectory">
        /// The base directory to reset and for aggregating report.
        /// </param>
        void Reset(DirectoryInfo baseDirectory);
    }
}
