/*
 * CStyleLineCounter.cs
 *
 * File Statistics
 * Version: 1.0
 *
 * Copyright (c) 2005, TopCoder, Inc. All rights reserved
 *
 * Authors: WishingBone, TCSDEVELOPER
 */

using System;
using System.Collections;
using System.IO;
using TopCoder.File.Statistics.Matcher;

namespace TopCoder.File.Statistics.Processor.LineCounter
{
    /// <summary>
    /// Counts lines for C style source files.
    /// </summary>
    ///
    /// <remarks>
    /// <para>
    /// This implementation might be able to handle C, C++, C#, or Java source code.
    /// However, it only takes opening braces and semicolons into account
    /// so it might not be very accurate in most cases,
    /// for example the statement <c>for (int i = 0; i &lt;= 5; i++) { test(); }</c>
    /// will be counted as 4 lines.
    /// Comments and string literals can be handled, so that opening braces and semicolons
    /// inside them are ignored as expected.
    /// It supports a single metric of "Line Count".
    /// It delegates configuration to base class without any addition.
    /// </para>
    /// <para>
    /// The code parser is implemented using a simple state machine.
    /// </para>
    /// </remarks>
    ///
    /// <threadsafety static="true" instance="true"/>
    public class CStyleLineCounter : FileProcessorBase
    {
        /// <summary>
        /// Implements a state machine that can process the source code.
        /// </summary>
        ///
        /// <threadsafety static="false" instance="false"/>
        private class SimpleStateMachine
        {
            /// <summary>
            /// Represents the state of the machine.
            /// </summary>
            private enum State
            {
                /// <summary>
                /// Not in comment or literal.
                /// </summary>
                StateStart,

                /// <summary>
                /// After a single slash
                /// (Input on slash would enter line comment state;
                /// input on star would enter block state).
                /// </summary>
                StateSlash,

                /// <summary>
                /// A star in a block comment (Input of slash would enter start state).
                /// </summary>
                StateStar,

                /// <summary>
                /// Back slash in single quote literal (next character is escaped).
                /// </summary>
                StateSQBackslash,

                /// <summary>
                /// Back slash in double quote literal (next character is escaped).
                /// </summary>
                StateDQBackslash,

                /// <summary>
                /// Single quote literal.
                /// </summary>
                StateSingleQuote,

                /// <summary>
                /// Double quote literal.
                /// </summary>
                StateDoubleQuote,

                /// <summary>
                /// Block comment.
                /// </summary>
                StateBlockComment,

                /// <summary>
                /// Line comment.
                /// </summary>
                StateLineComment
            }

            /// <summary>
            /// The current state of this machine.
            /// </summary>
            private State state = State.StateStart;

            /// <summary>
            /// The current line count. Initialized to 0 and increments whenever necessary.
            /// </summary>
            private long lines = 0;

            /// <summary>
            /// Gets the code line count so far.
            /// </summary>
            ///
            /// <value>
            /// A long representing the code line count so far.
            /// </value>
            public long Lines
            {
                get
                {
                    return lines;
                }
            }

            /// <summary>
            /// Initializes a new instance of the <see cref="SimpleStateMachine"/> class.
            /// </summary>
            public SimpleStateMachine()
            {
            }

            /// <summary>
            /// Processes a line.
            /// </summary>
            ///
            /// <param name="line">
            /// The line to process (don't include the new line character).
            /// </param>
            ///
            /// <exception cref="ArgumentNullException">
            /// <paramref name="line"/> is a null reference.
            /// </exception>
            public void Process(string line)
            {
                // cannot be null
                if (line == null)
                {
                    throw new ArgumentNullException("line");
                }
                // process each char
                foreach (char ch in line)
                {
                    Process(ch);
                }
                // add the new line char
                Process('\n');
            }

            /// <summary>
            /// Processes a character and updates state accordingly.
            /// </summary>
            ///
            /// <param name="ch">
            /// The character to process.
            /// </param>
            private void Process(char ch)
            {
                switch (state)
                {
                    // start state
                    case State.StateStart:
                        switch (ch)
                        {
                            case ';':
                            case '{':
                                lines++;
                                break;
                            case '/':
                                state = State.StateSlash;
                                break;
                            case '\'':
                                state = State.StateSingleQuote;
                                break;
                            case '"':
                                state = State.StateDoubleQuote;
                                break;
                        }
                        break;

                    // slash state
                    case State.StateSlash:
                        switch (ch)
                        {
                            case '*':
                                state = State.StateBlockComment;
                                break;
                            case '/':
                                state = State.StateLineComment;
                                break;
                            default:
                                state = State.StateStart;
                                break;
                        }
                        break;

                    // star state (star in block comment)
                    case State.StateStar:
                        switch (ch)
                        {
                            case '/':
                                state = State.StateStart;
                                break;
                            case '*':
                                state = State.StateStar;
                                break;
                            default:
                                state = State.StateBlockComment;
                                break;
                        }
                        break;

                    // SQ backslash state (backslash in single quote)
                    case State.StateSQBackslash:
                        state = State.StateSingleQuote;
                        break;

                    // DQ backslash state (backslash in double quote)
                    case State.StateDQBackslash:
                        state = State.StateDoubleQuote;
                        break;

                    // Single quote state
                    case State.StateSingleQuote:
                        switch (ch)
                        {
                            case '\\':
                                state = State.StateSQBackslash;
                                break;
                            case '\'':
                                state = State.StateStart;
                                break;
                        }
                        break;

                    // Double quote state
                    case State.StateDoubleQuote:
                        switch (ch)
                        {
                            case '\\':
                                state = State.StateDQBackslash;
                                break;
                            case '"':
                                state = State.StateStart;
                                break;
                        }
                        break;

                    // line comment state
                    case State.StateLineComment:
                        switch (ch)
                        {
                            case '\n':
                                state = State.StateStart;
                                break;
                        }
                        break;

                    // line comment state
                    case State.StateBlockComment:
                        switch (ch)
                        {
                            case '*':
                                state = State.StateStar;
                                break;
                        }
                        break;
                }
            }
        }


        /// <summary>
        /// The name of the metric used for this processor.
        /// </summary>
        public static readonly string Name = "Line Count";

        /// <summary>
        /// The display name of the metric used for this processor.
        /// </summary>
        public static readonly string DisplayName = "line";

        /// <summary>
        /// The description of the metric used for this processor.
        /// </summary>
        private static readonly string description = "Number of statements in a source code file.";

        /// <summary>
        /// The supported metric list for this processor.
        /// </summary>
        private static readonly Metric[] supportedMetric =
            { new Metric(Name, DisplayName, description) };

        /// <summary>
        /// Gets a list containing a single <see cref="Metric"/> composed of
        /// the <see cref="Name"/>, <see cref="DisplayName"/>, and a description.
        /// </summary>
        ///
        /// <value>
        /// A list containing a single <see cref="Metric"/> composed of
        /// the <see cref="Name"/>, <see cref="DisplayName"/>, and a description.
        /// </value>
        public override IList SupportedMetrics
        {
            get
            {
                // return the supported metric list
                return supportedMetric;
            }
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="CStyleLineCounter"/> instance
        /// with the specified alias name and <see cref="IFileMatcher"/>.
        /// </summary>
        ///
        /// <param name="alias">
        /// The alias name.
        /// </param>
        /// <param name="matcher">
        /// The <see cref="IFileMatcher"/> to use.
        /// </param>
        ///
        /// <exception cref="ArgumentNullException">
        /// Any of the argument is a null reference.
        /// </exception>
        /// <exception cref="ArgumentException">
        /// <paramref name="alias"/> is empty after trimmed.
        /// </exception>
        public CStyleLineCounter(string alias, IFileMatcher matcher)
            : base(alias, matcher)
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="CStyleLineCounter"/>
        /// which loads configuration from the specified nameSpace
        /// and prefix of the configuration.
        /// </summary>
        ///
        /// <remarks>
        /// The alias name will be loaded from the property <c>prefix + "alias"</c>,
        /// while the file matcher class will be loaded from the property
        /// <c>prefix + "matcher.classname"</c> passing the nameSpace and
        /// <c>prefix + "matcher."</c> to the file matcher constructor.
        /// If the property <c>prefix + "matcher.classname"</c> is missing,
        /// <see cref="AnyFileMatcher"/> will be used.
        /// </remarks>
        ///
        /// <param name="nameSpace">
        /// The namespace to load configuration values from.
        /// </param>
        /// <param name="prefix">
        /// The prefix of the property name to load configuration values from.
        /// </param>
        ///
        /// <exception cref="ArgumentNullException">
        /// Any of the argument is a null reference.
        /// </exception>
        /// <exception cref="ArgumentException">
        /// Any of the argument is empty after trimmed.
        /// </exception>
        /// <exception cref="ConfigurationException">
        /// A configuration error occurs, such as missing alias name,
        /// empty alias name, error instantiating file matcher.
        /// </exception>
        public CStyleLineCounter(string nameSpace, string prefix)
            : base(nameSpace, prefix)
        {
        }

        /// <summary>
        /// Count the lines in C style source code file.
        /// </summary>
        ///
        /// <remarks>
        /// This will assume that the specified file is a text file.
        /// It would open the file as a text stream and count the code lines until EOF is met.
        /// Code lines is only considered when a semicolon or opening brace is found.
        /// Semicolon or opening brace inside a string literal or comment sections are ignored.
        /// </remarks>
        ///
        /// <param name="file">
        /// The file to process.
        /// </param>
        ///
        /// <returns>
        /// A <see cref="StatsResult"/> representing the number of code lines in the file.
        /// </returns>
        ///
        /// <exception cref="ArgumentNullException">
        /// <paramref name="file"/> is a null reference.
        /// </exception>
        /// <exception cref="IOException">
        /// Any IO operation fails.
        /// </exception>
        /// <exception cref="FileProcessingException">
        /// The processor fails.
        /// </exception>
        public override StatsResult Process(FileInfo file)
        {
            // cannot be null
            if (file == null)
            {
                throw new ArgumentNullException("file");
            }

            try
            {
                // open as text and count the code line with simple state machine
                SimpleStateMachine machine = new SimpleStateMachine();
                using (StreamReader stream = file.OpenText())
                {
                    string line = stream.ReadLine();
                    while (line != null)
                    {
                        machine.Process(line);
                        line = stream.ReadLine();
                    }
                }

                // return the StatsResult
                StatsResult result = new StatsResult();
                result[(Metric) SupportedMetrics[0]] = machine.Lines;
                return result;
            }
            catch (IOException)
            {
                // just re-throw
                throw;
            }
            catch (Exception e)
            {
                // encapsulate all others in IOException
                throw new IOException(string.Format(
                    "Error processing file {0}", file.FullName), e);
            }

        }
    }
}
