/*
 * TextLineCounter.cs
 *
 * File Statistics
 * Version: 1.0
 *
 * Copyright (c) 2005, TopCoder, Inc. All rights reserved
 *
 * Authors: WishingBone, TCSDEVELOPER
 */

using System;
using System.Collections;
using System.IO;
using TopCoder.File.Statistics;
using TopCoder.File.Statistics.Matcher;

namespace TopCoder.File.Statistics.Processor.LineCounter
{
    /// <summary>
    /// Counts lines for text files.
    /// </summary>
    ///
    /// <remarks>
    /// It supports a single metric of "Line Count".
    /// It delegates configuration to base class without any addition.
    /// </remarks>
    ///
    /// <threadsafety static="true" instance="true"/>
    public class TextLineCounter : FileProcessorBase
    {
        /// <summary>
        /// The name of the metric used for this processor.
        /// </summary>
        public static readonly string Name = "Line Count";

        /// <summary>
        /// The display name of the metric used for this processor.
        /// </summary>
        public static readonly string DisplayName = "line";

        /// <summary>
        /// The description of the metric used for this processor.
        /// </summary>
        private static readonly string description = "Number of lines in a text file.";

        /// <summary>
        /// The supported metric list for this processor.
        /// </summary>
        private static readonly Metric[] supportedMetric =
            { new Metric(Name, DisplayName, description) };

        /// <summary>
        /// Gets a list containing a single <see cref="Metric"/> composed of
        /// the <see cref="Name"/>, <see cref="DisplayName"/>, and a description.
        /// </summary>
        ///
        /// <value>
        /// A list containing a single <see cref="Metric"/> composed of
        /// the <see cref="Name"/>, <see cref="DisplayName"/>, and a description.
        /// </value>
        public override IList SupportedMetrics
        {
            get
            {
                // return the supported metric list
                return supportedMetric;
            }
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="TextLineCounter"/> instance
        /// with the specified alias name and <see cref="IFileMatcher"/>.
        /// </summary>
        ///
        /// <param name="alias">
        /// The alias name.
        /// </param>
        /// <param name="matcher">
        /// The <see cref="IFileMatcher"/> to use.
        /// </param>
        ///
        /// <exception cref="ArgumentNullException">
        /// Any of the argument is a null reference.
        /// </exception>
        /// <exception cref="ArgumentException">
        /// <paramref name="alias"/> is empty after trimmed.
        /// </exception>
        public TextLineCounter(string alias, IFileMatcher matcher)
            : base(alias, matcher)
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="TextLineCounter"/>
        /// which loads configuration from the specified nameSpace
        /// and prefix of the configuration.
        /// </summary>
        ///
        /// <remarks>
        /// The alias name will be loaded from the property <c>prefix + "alias"</c>,
        /// while the file matcher class will be loaded from the property
        /// <c>prefix + "matcher.classname"</c> passing the nameSpace and
        /// <c>prefix + "matcher."</c> to the file matcher constructor.
        /// If the property <c>prefix + "matcher.classname"</c> is missing,
        /// <see cref="AnyFileMatcher"/> will be used.
        /// </remarks>
        ///
        /// <param name="nameSpace">
        /// The namespace to load configuration values from.
        /// </param>
        /// <param name="prefix">
        /// The prefix of the property name to load configuration values from.
        /// </param>
        ///
        /// <exception cref="ArgumentNullException">
        /// Any of the argument is a null reference.
        /// </exception>
        /// <exception cref="ArgumentException">
        /// Any of the argument is empty after trimmed.
        /// </exception>
        /// <exception cref="ConfigurationException">
        /// A configuration error occurs, such as missing alias name,
        /// empty alias name, error instantiating file matcher.
        /// </exception>
        public TextLineCounter(string nameSpace, string prefix)
            : base(nameSpace, prefix)
        {
        }

        /// <summary>
        /// Count the lines in the text file.
        /// </summary>
        ///
        /// <remarks>
        /// This will assume that the specified file is a text file.
        /// It would open the file as a text stream and count the lines until EOF is met.
        /// </remarks>
        ///
        /// <param name="file">
        /// The file to process.
        /// </param>
        ///
        /// <returns>
        /// A <see cref="StatsResult"/> representing the number of lines in the file.
        /// </returns>
        ///
        /// <exception cref="ArgumentNullException">
        /// <paramref name="file"/> is a null reference.
        /// </exception>
        /// <exception cref="IOException">
        /// Any IO operation fails.
        /// </exception>
        /// <exception cref="FileProcessingException">
        /// The processor fails.
        /// </exception>
        public override StatsResult Process(FileInfo file)
        {
            // cannot be null
            if (file == null)
            {
                throw new ArgumentNullException("file");
            }

            try
            {
                // open as text and count the line
                long count = 0;
                using (StreamReader stream = file.OpenText())
                {
                    while (stream.ReadLine() != null)
                    {
                        count++;
                    }
                }

                // return the StatsResult
                StatsResult result = new StatsResult();
                result[(Metric) SupportedMetrics[0]] = count;
                return result;
            }
            catch (IOException)
            {
                // just re-throw
                throw;
            }
            catch (Exception e)
            {
                // encapsulate all others in IOException
                throw new IOException(string.Format(
                    "Error processing file {0}", file.FullName), e);
            }
        }
    }
}
