/*
 * BasicStatsReporting.cs
 *
 * File Statistics
 * Version: 1.0
 *
 * Copyright (c) 2005, TopCoder, Inc. All rights reserved
 *
 * Authors: WishingBone, TCSDEVELOPER
 */

using System;
using System.Collections;
using System.Text;
using TopCoder.File.Statistics;

namespace TopCoder.File.Statistics.Reporting
{
    /// <summary>
    /// A reporting implementation that renders a user-friendly report.
    /// </summary>
    ///
    /// <remarks>
    /// Use <see cref="XmlStatsReporting"/> if you need to process the statistic results.
    /// This implementation does not need any configuration.
    /// The constructor with nameSpace and prefix arguments is provided
    /// for reflection purpose only.
    /// </remarks>
    ///
    /// <threadsafety static="false" instance="false"/>
    public class BasicStatsReporting : AbstractStatsReporting
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="BasicStatsReporting"/> class.
        /// </summary>
        public BasicStatsReporting()
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="BasicStatsReporting"/> class
        /// with the specified namespace and prefix.
        /// </summary>
        ///
        /// <remarks>
        /// This constructor is provided for reflection purpose only.
        /// No work is actually performed by this constructor as
        /// no configuration is needed by this class.
        /// </remarks>
        ///
        /// <param name="nameSpace">
        /// Not applicable.
        /// </param>
        /// <param name="prefix">
        /// Not applicable.
        /// </param>
        ///
        /// <exception cref="ArgumentNullException">
        /// Any of the argument is a null reference.
        /// </exception>
        /// <exception cref="ArgumentException">
        /// <paramref name="nameSpace"/> is empty.
        /// </exception>
        public BasicStatsReporting(string nameSpace, string prefix)
            : base(nameSpace, prefix)
        {
        }

        /// <summary>
        /// Generates a user-friendly report for all the aggregated results.
        /// </summary>
        ///
        /// <returns>
        /// The generated report.
        /// </returns>
        ///
        /// <exception cref="StatsReportingException">
        /// An error occurs.
        /// </exception>
        public override string GenerateReport()
        {
            StringBuilder result = new StringBuilder();

            try
            {
                // header
                string title = string.Format("File Statistics Report For {0}",
                    ((BaseDirectory == null) ? "\\" : BaseDirectory.FullName));

                result.Append(title).Append(Environment.NewLine);
                result.Append('=', title.Length).Append(Environment.NewLine).Append(Environment.NewLine);

                // write each section
                AddSection(result, "Per file statistic", FileStats);
                AddSection(result, "Per directory statistic", DirectoryStats);
                AddSection(result, "Per alias statistic", AliasStats);

                // overall
                title = "Overall statistic";
                result.Append(title).Append(Environment.NewLine);
                result.Append('-', title.Length).Append(Environment.NewLine);
                AddMetric(result, OverallStats);
                result.Append(Environment.NewLine);

                // footer: write metric legend
                // the overall should have all the metric, so we'd use that
                AddMetricLegend(result, OverallStats);

                // return the string result
                return result.ToString();
            }
            catch (Exception e)
            {
                throw new StatsReportingException("Error generating basic report", e);
            }
        }

        /// <summary>
        /// Writes a file/directory/alias section of the statistic.
        /// </summary>
        ///
        /// <param name="output">
        /// The <see cref="StringBuilder"/> instance to append to.
        /// </param>
        /// <param name="title">
        /// The title of this section.
        /// </param>
        /// <param name="result">
        /// The <see cref="StatsCollection"/> to write.
        /// </param>
        private void AddSection(StringBuilder output, string title, StatsCollection result)
        {
            // title
            output.Append(title).Append(Environment.NewLine);
            output.Append('-', title.Length).Append(Environment.NewLine);

            // print the key
            IList keys = result.AllKeys;
            foreach (string key in keys)
            {
                output.Append(key).Append(Environment.NewLine);

                // write all metrics for this key
                AddMetric(output, result[key]);
            }
            output.Append(Environment.NewLine);
        }

        /// <summary>
        /// Writes each metric (indented) and its value.
        /// </summary>
        ///
        /// <param name="output">
        /// The <see cref="StringBuilder"/> instance to append to.
        /// </param>
        /// <param name="result">
        /// The StatsResult containing Metrics to write.
        /// </param>
        private void AddMetric(StringBuilder output, StatsResult result)
        {
            // iterate each metric
            IList metrics = result.Metrics;
            foreach (Metric metric in metrics)
            {
                // write the metric name then it's value (indented)
                output.AppendFormat("    {0}: {1}",
                    metric.Name, result[metric]).Append(Environment.NewLine);
            }
        }

        /// <summary>
        /// Writes each metric name, display name, and its description as a legend.
        /// </summary>
        ///
        /// <param name="output">
        /// The <see cref="StringBuilder"/> instance to append to.
        /// </param>
        /// <param name="result">
        /// The StatsResult containing Metrics information.
        /// </param>
        private void AddMetricLegend(StringBuilder output, StatsResult result)
        {
            string title = "Metric legend";
            output.Append(title).Append(Environment.NewLine);
            output.Append('-', title.Length).Append(Environment.NewLine);

            // iterate each metric
            IList metrics = result.Metrics;
            foreach (Metric metric in metrics)
            {
                // write the metric name, display name, and description
                output.AppendFormat("{0} ({1}): {2}",
                    metric.Name, metric.DisplayName, metric.Description).Append(Environment.NewLine);
            }
        }
    }
}
