/*
 * TestRegexMatcher.cs
 *
 * File Statistics
 * Version: 1.0
 *
 * Copyright (c) 2005, TopCoder, Inc. All rights reserved
 *
 * Author: TCSDEVELOPER
 */

using System;
using System.Collections;
using System.IO;
using System.Text.RegularExpressions;
using NUnit.Framework;
using TopCoder.File.Statistics.Tests;

namespace TopCoder.File.Statistics.Matcher.Tests
{
    /// <summary>
    /// Unit tests for the <see cref="RegexMatcher"/> class.
    /// </summary>
    [TestFixture]
    public class TestRegexMatcher : TestHelper
    {
        /// <summary>
        /// The <see cref="RegexMatcher"/> instance to test.
        /// Initialized in each unit test case according to its need.
        /// </summary>
        private IFileMatcher matcher;

        /// <summary>
        /// A list of regular expressions to match.
        /// Initialized in SetUp() and used to test constructor(IList).
        /// </summary>
        private IList regexs;

        /// <summary>
        /// A regex to match a filename without extension (filename only (no dot))
        /// </summary>
        private Regex NoExtensionRegex = new Regex(@"^[^.]+$");

        /// <summary>
        /// Initializes the <see cref="RegexMatcher"/> instance to test
        /// using the test_files/Matcher_Regex.xml.
        /// </summary>
        [SetUp]
        public void SetUp()
        {
            // load test_files/Matcher_Regex.xml
            // used for constructor with nameSpace and prefix argument
            base.LoadConfigFile(@"..\..\test_files\Matcher_Regex.xml");

            // initialize regexs list (with *.txt (case-insensitive) and *.[a-h])
            regexs = new ArrayList();
            regexs.Add(new Regex(@".*\.txt", RegexOptions.IgnoreCase));
            regexs.Add(new Regex(@".*\.[a-h]"));
        }

        /// <summary>
        /// Tears down configuration manager.
        /// </summary>
        [TearDown]
        public void TearDown()
        {
            manager.Clear();
        }

        /// <summary>
        /// Tests the static property key name.
        /// </summary>
        [Test]
        public void TestStaticPropertyKey()
        {
            Assert.AreEqual("patterns", RegexMatcher.PatternsPropertyKey,
                "The property key name is wrong");
        }

        /// <summary>
        /// Tests constructor(string) is initialized with a single regex
        /// and Match() only returns true for matched file names.
        /// </summary>
        [Test]
        public void TestConstructor1()
        {
            // txt (case-insensitive)
            matcher = new RegexMatcher(new Regex(@".*\.txt", RegexOptions.IgnoreCase));

            // .txt, .TxT should match
            Assert.IsTrue(matcher.Match(new FileInfo(SampleText)),
                "txt should match");
            Assert.IsTrue(matcher.Match(new FileInfo("test.TxT")),
                "TxT should match (case-insensitive)");

            // should not match .A-.Z
            for (char ch = 'A'; ch <= 'Z'; ch++)
            {
                Assert.IsFalse(matcher.Match(new FileInfo("test." + ch)),
                    ch + " should not match");
            }

            // should not match file without extension
            Assert.IsFalse(matcher.Match(new FileInfo("test")),
                "File without extension should not match");
        }

        /// <summary>
        /// Tests constructor(string) is initialized with a no-extension regex
        /// and Match() returns true only for file with no extension.
        /// </summary>
        [Test]
        public void TestConstructor1NoExtension()
        {
            // no extension
            matcher = new RegexMatcher(NoExtensionRegex);

            // .txt, .TXT should not match
            Assert.IsFalse(matcher.Match(new FileInfo(SampleText)),
                "txt should not match");
            Assert.IsFalse(matcher.Match(new FileInfo("test.TxT")),
                "TxT should not match (case-insensitive)");

            // should not match .A-.Z
            for (char ch = 'A'; ch <= 'Z'; ch++)
            {
                Assert.IsFalse(matcher.Match(new FileInfo("test." + ch)),
                    ch + " should not match");
            }

            // should match file without extension
            Assert.IsTrue(matcher.Match(new FileInfo("test")),
                "File without extension should match");
        }

        /// <summary>
        /// Tests constructor(string) throws ArgumentNullException
        /// given null argument.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestConstructor1Null()
        {
            new RegexMatcher((Regex) null);
        }

        /// <summary>
        /// Tests constructor(IList) is initialized with two regexs
        /// and Match() only returns true for matched file names.
        /// </summary>
        [Test]
        public void TestConstructor2()
        {
            // *.txt (case-insensitive) and *.[a-h]
            matcher = new RegexMatcher(regexs);

            // .txt, .TxT should match
            Assert.IsTrue(matcher.Match(new FileInfo(SampleText)),
                "txt should match");
            Assert.IsTrue(matcher.Match(new FileInfo("test.TxT")),
                "TxT should match (case-insensitive)");

            // should match only .a-.h (case-sensitive)
            for (char ch = 'A'; ch <= 'Z'; ch++)
            {
                // test with uppercase extension
                Assert.IsFalse(matcher.Match(new FileInfo("test." + ch)),
                    ch + " should not match");

                // test with lowercase extension
                char chLower = Char.ToLower(ch);

                // only .a-.h should match
                if ((chLower >= 'a') && (chLower <= 'h'))
                {
                    Assert.IsTrue(matcher.Match(new FileInfo("test." + chLower)),
                        chLower + " should match");
                }
                else
                {
                    Assert.IsFalse(matcher.Match(new FileInfo("test." + chLower)),
                        chLower + " should not match");
                }
            }

            // should not match file without extension
            Assert.IsFalse(matcher.Match(new FileInfo("test")),
                "File without extension should not match");
        }

        /// <summary>
        /// Tests constructor(IList) is initialized with two regexs + no extension regex
        /// and Match() returns true only for matched file names
        /// and file without extension.
        /// </summary>
        [Test]
        public void TestConstructor2Empty()
        {
            // add no-extension regex
            regexs.Add(NoExtensionRegex);
            matcher = new RegexMatcher(regexs);

            // .txt, .TxT, and .c should match
            Assert.IsTrue(matcher.Match(new FileInfo(SampleText)),
                "txt should match");
            Assert.IsTrue(matcher.Match(new FileInfo("test.TxT")),
                "TxT should match (case-insensitive)");
            Assert.IsTrue(matcher.Match(new FileInfo("test.c")),
                "c should match");

            // should match only .a-.h (case-sensitive)
            for (char ch = 'A'; ch <= 'Z'; ch++)
            {
                // test with uppercase extension
                Assert.IsFalse(matcher.Match(new FileInfo("test." + ch)),
                    ch + " should not match");

                // test with lowercase extension
                char chLower = Char.ToLower(ch);

                // only .a-.h should match
                if ((chLower >= 'a') && (chLower <= 'h'))
                {
                    Assert.IsTrue(matcher.Match(new FileInfo("test." + chLower)),
                        chLower + " should match");
                }
                else
                {
                    Assert.IsFalse(matcher.Match(new FileInfo("test." + chLower)),
                        chLower + " should not match");
                }
            }

            // should match file without extension
            Assert.IsTrue(matcher.Match(new FileInfo("test")),
                "File without extension should match");
        }

        /// <summary>
        /// Tests constructor(IList) throws ArgumentNullException
        /// given null argument.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestConstructor2Null()
        {
            new RegexMatcher((IList) null);
        }

        /// <summary>
        /// Tests constructor(IList) throws ArgumentException
        /// given empty list.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestConstructor2List()
        {
            new RegexMatcher(new ArrayList());
        }

        /// <summary>
        /// Tests constructor(IList) throws ArgumentException
        /// given list containing null element.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestConstructor2NullElement()
        {
            regexs.Add(null);
            new RegexMatcher(regexs);
        }

        /// <summary>
        /// Tests constructor(IList) throws ArgumentException
        /// given list containing non Regex element.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestConstructor2NonRegexElement()
        {
            regexs.Add("test");
            new RegexMatcher(regexs);
        }

        /// <summary>
        /// Tests constructor(string, string) is initialized with two regexs
        /// and Match() only returns true for matched file names.
        /// </summary>
        [Test]
        public void TestConstructor3()
        {
            // initialize RegexMatcher (only .txt and .[t-z])
            matcher = new RegexMatcher(ValidNamespace, "matcher.");

            // .txt should match, .TxT should not
            Assert.IsTrue(matcher.Match(new FileInfo(SampleText)),
                "txt should match");
            Assert.IsFalse(matcher.Match(new FileInfo("test.TxT")),
                "TxT should not match (case-sensitive)");

            // should match only .t-.z (case-sensitive)
            for (char ch = 'A'; ch <= 'Z'; ch++)
            {
                // test with uppercase extension
                Assert.IsFalse(matcher.Match(new FileInfo("test." + ch)),
                    ch + " should not match");

                // test with lowercase extension
                char chLower = Char.ToLower(ch);

                // only .t-.z should match
                if ((chLower >= 't') && (chLower <= 'z'))
                {
                    Assert.IsTrue(matcher.Match(new FileInfo("test." + chLower)),
                        chLower + " should match");
                }
                else
                {
                    Assert.IsFalse(matcher.Match(new FileInfo("test." + chLower)),
                        chLower + " should not match");
                }
            }

            // should not match file without extension
            Assert.IsFalse(matcher.Match(new FileInfo("test")),
                "File without extension should not match");
        }

        /// <summary>
        /// Tests constructor(string, string) is initialized with no extension regex
        /// and Match() returns true only for file without extension.
        /// </summary>
        [Test]
        public void TestConstructor3NoExtension()
        {
            // initialize RegexMatcher (only no extension)
            matcher = new RegexMatcher(ValidNamespace, "matcher_noextension.");

            // .txt, .TxT, and .c should not match
            Assert.IsFalse(matcher.Match(new FileInfo(SampleText)),
                "txt should not match");
            Assert.IsFalse(matcher.Match(new FileInfo("test.TxT")),
                "TxT should not match (case-insensitive)");
            Assert.IsFalse(matcher.Match(new FileInfo("test.c")),
                "c should not match");

            // should not match .A-.Z, nor .a-.z
            for (char ch = 'A'; ch <= 'Z'; ch++)
            {
                // uppercase version
                Assert.IsFalse(matcher.Match(new FileInfo("test." + ch)),
                    ch + " should not match");

                // lowercase version
                char chLower = Char.ToLower(ch);
                Assert.IsFalse(matcher.Match(new FileInfo("test." + chLower)),
                    chLower + " should not match");
            }

            // should match file without extension
            Assert.IsTrue(matcher.Match(new FileInfo("test")),
                "File without extension should match");
        }

        /// <summary>
        /// Tests constructor(string, string) is initialized with empty string regex
        /// and it won't match anything.
        /// </summary>
        [Test]
        public void TestConstructor3EmptyRegex()
        {
            // initialize RegexMatcher (empty regex)
            matcher = new RegexMatcher(ValidNamespace, "matcher_empty.");

            // .txt, .TxT, and .c should not match
            Assert.IsFalse(matcher.Match(new FileInfo(SampleText)),
                "txt should not match");
            Assert.IsFalse(matcher.Match(new FileInfo("test.TxT")),
                "TxT should not match (case-insensitive)");
            Assert.IsFalse(matcher.Match(new FileInfo("test.c")),
                "c should not match");

            // should not match .A-.Z, nor .a-.z
            for (char ch = 'A'; ch <= 'Z'; ch++)
            {
                // uppercase version
                Assert.IsFalse(matcher.Match(new FileInfo("test." + ch)),
                    ch + " should not match");

                // lowercase version
                char chLower = Char.ToLower(ch);
                Assert.IsFalse(matcher.Match(new FileInfo("test." + chLower)),
                    chLower + " should not match");
            }

            // should not match file without extension
            Assert.IsFalse(matcher.Match(new FileInfo("test")),
                "File without extension should match");
        }

        /// <summary>
        /// Tests constructor(string, string) throws ArgumentNullException
        /// given null nameSpace.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestConstructor3NullNamespace()
        {
            new RegexMatcher(null, "matcher.");
        }

        /// <summary>
        /// Tests constructor(string, string) throws ArgumentNullException
        /// given null prefix.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestConstructor3NullPrefix()
        {
            new RegexMatcher(ValidNamespace, null);
        }

        /// <summary>
        /// Tests constructor(string, string) throws ArgumentException
        /// given empty namespace.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestConstructor3EmptyNamespace()
        {
            new RegexMatcher(" ", "matcher.");
        }

        /// <summary>
        /// Tests constructor(string, string) throws ConfigurationException
        /// given non-existing prefix.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ConfigurationException))]
        public void TestConstructor3NotExist()
        {
            new RegexMatcher(ValidNamespace, "notexists.");
        }

        /// <summary>
        /// Tests constructor(string, string) throws ConfigurationException
        /// given non-existing empty prefix.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ConfigurationException))]
        public void TestConstructor3NotExistEmpty()
        {
            new RegexMatcher(ValidNamespace, "");
        }

        /// <summary>
        /// Tests constructor(string, string) throws ConfigurationException
        /// given non-existing empty-space prefix.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ConfigurationException))]
        public void TestConstructor3NotExistEmptySpace()
        {
            new RegexMatcher(ValidNamespace, " ");
        }

        /// <summary>
        /// Tests constructor(string, string) throws ConfigurationException
        /// given regex which won't compile (missing closing parentheses).
        /// </summary>
        [Test]
        [ExpectedException(typeof(ConfigurationException))]
        public void TestConstructor3InvalidRegex()
        {
            new RegexMatcher(ValidNamespace, "matcher_invalidregex.");
        }

        /// <summary>
        /// Tests constructor(string, string) throws ConfigurationException
        /// given no list.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ConfigurationException))]
        public void TestConstructor3NoList()
        {
            new RegexMatcher(ValidNamespace, "matcher_invalid.");
        }

        /// <summary>
        /// Tests Match() throws ArgumentNullException
        /// given null FileInfo.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestMatchNull()
        {
            matcher.Match(null);
        }
    }
}
