/*
 * ArgumentOption.cs
 *
 * File Statistics
 * Version: 1.0
 *
 * Copyright (c) 2005, TopCoder, Inc. All rights reserved
 *
 * Authors: WishingBone, TCSDEVELOPER
 */

using System;
using TopCoder.Util.DataValidator;

namespace TopCoder.File.Statistics
{
    /// <summary>
    /// A pure data oriented class which packs together the switch name and help message,
    /// required flag and an optional validator for an option.
    /// </summary>
    ///
    /// <remarks>
    /// An <see cref="IArgumentHandler"/> can propose multiple options to the command
    /// line interface.
    /// </remarks>
    ///
    /// <threadsafety static="true" instance="true"/>
    public class ArgumentOption
    {
        /// <summary>
        /// The switch name for this option. Swith name unique identifies an argument.
        /// But switch name does not need to be unique among all proposals
        /// because the arguments could be shared (for instance we are very likely to
        /// configure a IFileMatcher twice).
        /// Initialized in ctor().
        /// </summary>
        private string switchName;

        /// <summary>
        /// This is the helper message to be printed to the user.
        /// It's free style text but should normally respect the format
        /// of the help message screen of the respective implementation.
        /// Initialized in ctor.
        /// </summary>
        private string helpMessage;

        /// <summary>
        /// A flag indicates whether the switch is required or not.
        /// Notice it is only concerned with the switch rather than
        /// any actual value after the switch.
        /// Initialized in ctor.
        /// </summary>
        private bool required;

        /// <summary>
        /// An optional validator to validate the argument value at the command line interface level.
        /// This validation is not required and IArgumentHandler can perform more validation
        /// when the arguments are received.
        /// </summary>
        private IValidator validator;

        /// <summary>
        /// Gets the switch name.
        /// </summary>
        ///
        /// <value>
        /// The switch name.
        /// </value>
        public string SwitchName
        {
            get
            {
                return switchName;
            }
        }

        /// <summary>
        /// Gets the help message for this option.
        /// </summary>
        ///
        /// <value>
        /// The help message for this option.
        /// </value>
        public string HelpMessage
        {
            get
            {
                return helpMessage;
            }
        }

        /// <summary>
        /// Gets the required flag of this option.
        /// </summary>
        ///
        /// <value>
        /// A boolean value indicating the required flag of this option.
        /// </value>
        public bool IsRequired
        {
            get
            {
                return required;
            }
        }

        /// <summary>
        /// Gets the validator for this option.
        /// </summary>
        ///
        /// <value>
        /// A <see cref="IValidator"/> instance or <b>null</b> if no validator is specified.
        /// </value>
        public IValidator Validator
        {
            get
            {
                return validator;
            }
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="ArgumentOption"/> class.
        /// </summary>
        ///
        /// <remarks>
        /// The <see cref="Validator"/> will be <b>null</b>.
        /// </remarks>
        ///
        /// <param name="switchName">
        /// The name of the switch.
        /// </param>
        /// <param name="helpMessage">
        /// The help message for the switch.
        /// </param>
        /// <param name="required">
        /// A boolean value indicating whether this switch is required.
        /// </param>
        ///
        /// <exception cref="ArgumentNullException">
        /// Any of the string argument is a null reference.
        /// </exception>
        /// <exception cref="ArgumentException">
        /// Any of the string argument is empty after trimmed.
        /// </exception>
        public ArgumentOption(string switchName, string helpMessage, bool required)
        {
            // cannot be null
            if (switchName == null)
            {
                throw new ArgumentNullException("switchName");
            }
            if (helpMessage == null)
            {
                throw new ArgumentNullException("helpMessage");
            }
            // cannot be empty
            if (switchName.Trim().Length == 0)
            {
                throw new ArgumentException("'switchName' cannot be empty", "switchName");
            }
            if (helpMessage.Trim().Length == 0)
            {
                throw new ArgumentException("'helpMessage' cannot be empty", "helpMessage");
            }

            // save to instance members
            this.switchName = switchName;
            this.helpMessage = helpMessage;
            this.required = required;
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="ArgumentOption"/> class
        /// with the specified arguments.
        /// </summary>
        ///
        /// <param name="switchName">
        /// The name of the switch.
        /// </param>
        /// <param name="helpMessage">
        /// The help message for the switch.
        /// </param>
        /// <param name="required">
        /// A boolean value indicating whether this switch is required.
        /// </param>
        /// <param name="validator">
        /// The switch validator to use.
        /// </param>
        ///
        /// <exception cref="ArgumentNullException">
        /// Any of the string argument or validator is a null reference.
        /// </exception>
        /// <exception cref="ArgumentException">
        /// Any of the string argument is empty after trimmed.
        /// </exception>
        public ArgumentOption(string switchName, string helpMessage, bool required, IValidator validator)
            : this(switchName, helpMessage, required)
        {
            // cannot be null
            if (validator == null)
            {
                throw new ArgumentNullException("validator");
            }

            // save to instance members
            this.validator = validator;
        }
    }
}
