/*
 * IFileProcessor.cs
 *
 * File Statistics
 * Version: 1.0
 *
 * Copyright (c) 2005, TopCoder, Inc. All rights reserved
 *
 * Authors: WishingBone, TCSDEVELOPER
 */

using System;
using System.IO;
using System.Collections;
using TopCoder.File.Statistics.Reporting;
using TopCoder.Util.ConfigurationManager;

namespace TopCoder.File.Statistics
{
    /// <summary>
    /// Provides the main API to process a file and get its statistic.
    /// </summary>
    ///
    /// <remarks>
    /// <para>
    /// It also includes APIs for maintaining a list of file processors,
    /// a reporting handler, and controls the way to process the file.
    /// Files are processed one by one and the results are aggregated into the reporting.
    /// If you want to start a new reporting you should either explicitly set the reporting
    /// or get the old reporting to reset it.
    /// </para>
    /// <para>
    /// There are two ways to process a file:
    /// <list type="bullet">
    /// <item>
    ///     <description>
    ///     Process it with the first matching processor.
    ///     This class maintains the file processor order as it is added/removed/changed.
    ///     This order is important when determining the first matched processor
    ///     if the matchall flag is false.
    ///     </description>
    /// </item>
    /// <item>
    ///     <description>Process it with each matching processor.</description>
    /// </item>
    /// </list>
    /// </para>
    /// Refer to <see cref="ExtendedFileStatistics"/> for advanced usage.
    /// <para>
    /// This class can be initialized from a configuration namespace which should be preloaded.
    /// The following properties are processed:
    /// <list type="bullet">
    /// <item>
    ///     <term>processors.names</term>
    ///     <description>
    ///     A multiple values configuration describing the property prefix for configured processors.
    ///     This property is required and cannot be empty.
    ///     </description>
    /// </item>
    /// <item>
    ///     <term>[processornames].classname</term>
    ///     <description>
    ///     The fully qualified class name of an <see cref="IFileProcessor"/> implementation
    ///     which accepts a nameSpace and prefix string as its constructor argument.
    ///     The prefix <c>[processornames] + "."</c> will be passed to the constructor.
    ///     This property is required.
    ///     </description>
    /// </item>
    /// <item>
    ///     <term>reporting.classname</term>
    ///     <description>
    ///     The fully qualified class name of an <see cref="IStatsReporting"/> implementation
    ///     which accepts nameSpace and prefix string as its constructor arguments.
    ///     The prefix <c>"reporting."</c> will be passed to the constructor.
    ///     This property is optional and <see cref="XmlStatsReporting"/> is the default value.
    ///     </description>
    /// </item>
    /// <item>
    ///     <term>matchall</term>
    ///     <description>
    ///     A boolean flag that indicates whether all matching processors or the first matching processor
    ///     will be used to process the file.
    ///     It is regarded <b>true</b> when value is either "on", "yes", "true" case-insensitively.
    ///     This property is optional and default to false.
    ///     </description>
    /// </item>
    /// </list>
    /// </para>
    /// <para>
    /// Since version 2.0b this class adds a package private method
    /// to return all the <see cref="IArgumentHandler"/> to the command line interface.
    /// </para>
    /// </remarks>
    ///
    /// <example>
    /// The following code shows how to generate a statistic for a file:
    /// <code>
    /// // initialize FileStatistics
    /// FileStatistics statistics = new FileStatistics(NAMESPACE);
    ///
    /// // create a .txt file matcher
    /// IFileMatcher matcher = new ExtensionMatcher("txt");
    ///
    /// // create the text line counter file processor and add it
    /// IFileProcessor processor = new TextLineCounter("Text File", matcher);
    /// statistics.AddFileProcessor(processor);
    ///
    /// // create stats reporting implementation
    /// IStatsReporting reporting = new BasicStatsReporting();
    /// statistics.Reporting = reporting;
    ///
    /// // process single file
    /// statistics.ProcessFile(new FileInfo("sample.txt")));
    ///
    /// // obtain report
    /// string report = statistics.Reporting.GenerateReport();
    /// </code>
    /// </example>
    ///
    /// <threadsafety static="false" instance="false"/>
    public class FileStatistics
    {
        /// <summary>
        /// The property key name prefix for the list of configured processors.
        /// </summary>
        ///
        /// <remarks>
        /// The list of processors should be listed with the key <c>"processors.names"</c>.
        /// The actual processor is defined in <c>"processors.classname"</c>.
        /// When the implementation is instantiated, the property itself is passed in
        /// as the prefix in the constructor argument (<c>"processors."</c>).
        /// </remarks>
        public static readonly string ProcessorsPropertyKey = "processors";

        /// <summary>
        /// The property key name prefix for the <see cref="IStatsReporting"/> implementation.
        /// </summary>
        ///
        /// <remarks>
        /// The reporting class should be listed with the key <c>"reporting.classname"</c>.
        /// When the implementation is instantiated, the property itself is passed in
        /// as the prefix in the constructor argument (<c>"reporting."</c>).
        /// </remarks>
        public static readonly string ReportingPropertyKey = "reporting";

        /// <summary>
        /// The property key name which specifies whether a file is processed by the first matching processor
        /// or all of the matching processor.
        /// </summary>
        ///
        /// <remarks>
        /// This property is optional (default to <b>false</b>).
        /// Its value is considered true if it's "true", "yes" or "on" case-insensitively.
        /// </remarks>
        public static readonly string MatchAllPropertyKey = "matchall";

        /// <summary>
        /// A sub-property in "[processorsname]" and "reporting"
        /// to specify the fully qualified class name of
        /// the <see cref="IFileProcessor"/> or <see cref="IStatsReporting"/> implementation
        /// that owns a constructor with nameSpace and prefix arguments.
        /// </summary>
        public static readonly string ClassnamePropertyKey = "classname";

        /// <summary>
        /// A sub-property in "processors"
        /// to specify the list of available processors prefix configuration.
        /// </summary>
        private static readonly string namesPropertyKey = "names";

        /// <summary>
        /// The processors aggregated.
        /// We need to maintain the processor order, hence IList is used, instead of IDictionary.
        /// This is initialized to an ArrayList and the reference will never change.
        /// The ArrayList will contain instance of IFileProcessor.
        /// </summary>
        private IList processors = new ArrayList();

        /// <summary>
        /// The reporting to use. This is mutable.
        /// Initialized by default to XmlStatsReporting.
        /// </summary>
        private IStatsReporting reporting = new XmlStatsReporting();

        /// <summary>
        /// A boolean flag whether to process the file with all matching processor.
        /// Default to false.
        /// </summary>
        private bool matchAll = false;

        /// <summary>
        /// Get a list of all the processors.
        /// </summary>
        ///
        /// <value>
        /// An <see cref="IList"/> containing
        /// the registered <see cref="IFileProcessor"/> instances.
        /// </value>
        public IList AllFileProcessors
        {
            get
            {
                // return a clone
                return new ArrayList(this.processors);
            }
        }

        /// <summary>
        /// Gets or sets the reporting instance.
        /// </summary>
        ///
        /// <value>
        /// The <see cref="IStatsReporting"/> instance used.
        /// </value>
        ///
        /// <exception cref="ArgumentNullException">
        /// Setting to a <b>null</b> value.
        /// </exception>
        public IStatsReporting Reporting
        {
            get
            {
                return this.reporting;
            }
            set
            {
                // cannot be null
                if (value == null)
                {
                    throw new ArgumentNullException("Reporting");
                }

                this.reporting = value;
            }
        }

        /// <summary>
        /// Gets or sets a boolean value indicating whether to process the file
        /// with all matching processor.
        /// </summary>
        ///
        /// <value>
        /// <b>true</b> if the file should be processed with all matching processor,
        /// or <b>false</b> if the file should be processed with the first
        /// matching processor.
        /// </value>
        public bool IsMatchAll
        {
            get
            {
                return this.matchAll;
            }
            set
            {
                this.matchAll = value;
            }
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="FileStatistics"/> class.
        /// </summary>
        public FileStatistics()
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="FileStatistics"/> class
        /// using configuration values from the specified nameSpace.
        /// </summary>
        ///
        /// <remarks>
        /// The namespace should be already loaded.
        /// It loads the configured processors, the reporting, as well as the matchall flag.
        /// </remarks>
        ///
        /// <param name="nameSpace">
        /// The namespace to load configuration values from.
        /// </param>
        ///
        /// <exception cref="ArgumentNullException">
        /// <paramref name="nameSpace"/> is a null reference.
        /// </exception>
        /// <exception cref="ArgumentException">
        /// <paramref name="nameSpace"/> is empty after trimmed.
        /// </exception>
        /// <exception cref="ConfigurationException">
        /// A config error occurs.
        /// </exception>
        public FileStatistics(string nameSpace)
        {
            // cannot be null nor empty
            if (nameSpace == null)
            {
                throw new ArgumentNullException("nameSpace");
            }
            if (nameSpace.Trim().Length == 0)
            {
                throw new ArgumentException("'nameSpace' cannot be empty", "nameSpace");
            }

            // process config
            try
            {
                // load processors
                LoadProcessors(nameSpace);

                // load reporting instance
                LoadReporting(nameSpace);

                // load matchall flag
                LoadMatchAll(nameSpace);
            }
            catch (ConfigurationException)
            {
                // just re-throw
                throw;
            }
            catch (Exception e)
            {
                // encapsulate all others in ConfigurationException
                throw new ConfigurationException(string.Format(
                    "Error initializing using namespace {0}", nameSpace), e);
            }
        }

        /// <summary>
        /// Adds a file processor to the aggregation.
        /// </summary>
        ///
        /// <remarks>
        /// The class maintains the processor order internally as processor is added/removed/changed.
        /// The specified processor will be added at the end of the list.
        /// If there is a processor with the same alias,
        /// it'd be removed first and this new processor is added at the end of the list.
        /// </remarks>
        ///
        /// <param name="processor">
        /// The file processor to add.
        /// </param>
        ///
        /// <exception cref="ArgumentNullException">
        /// <paramref name="processor"/> is a null reference.
        /// </exception>
        public void AddFileProcessor(IFileProcessor processor)
        {
            // cannot be null
            if (processor == null)
            {
                throw new ArgumentNullException("processor");
            }

            // remove processor with the same alias, if exists
            RemoveFileProcessor(processor.Alias);

            // add this processor
            this.processors.Add(processor);
        }

        /// <summary>
        /// Removes the file processor with given alias.
        ///
        /// </summary>
        ///
        /// <remarks>
        /// If no such processor exists, nothing will happen and <b>null</b> is returned.
        /// </remarks>
        ///
        /// <param name="alias">
        /// The alias of the processor.
        /// </param>
        ///
        /// <returns>
        /// The processor just removed or <b>null</b> if no such processor exists.
        /// </returns>
        ///
        /// <exception cref="ArgumentNullException">
        /// <paramref name="alias"/> is a null reference.
        /// </exception>
        /// <exception cref="ArgumentException">
        /// <paramref name="alias"/> is empty after trimmed.
        /// </exception>
        public IFileProcessor RemoveFileProcessor(string alias)
        {
            // cannot be null nor empty
            if (alias == null)
            {
                throw new ArgumentNullException("alias");
            }
            if (alias.Trim().Length == 0)
            {
                throw new ArgumentException("'alias' cannot be empty", "alias");
            }

            // find the processor
            IFileProcessor processor = this[alias];
            // remove it if it exists (it's ok to call ArrayList.Remove(null))
            this.processors.Remove(processor);

            return processor;
        }

        /// <summary>
        /// Clears all the aggregated processors.
        /// </summary>
        public void ClearAllFileProcessors()
        {
            this.processors.Clear();
        }

        /// <summary>
        /// Processes a single file and aggregates the file statistic
        /// to the registered reporting instance.
        /// </summary>
        ///
        /// <remarks>
        /// This will determines, in order, if each registered processor matches (could handle)
        /// the file. If it does, the file processor is executed and statistic is aggregated
        /// to the reporting instance.
        /// This process is repeated with all the rest of the registered processors,
        /// unless matchall flag is false, in which case only the first matched processor
        /// is executed.
        /// </remarks>
        ///
        /// <param name="file">
        /// The file to process.
        /// </param>
        ///
        /// <exception cref="ArgumentNullException">
        /// <paramref name="file"/> is a null reference.
        /// </exception>
        /// <exception cref="IOException">
        /// Any IO operation fails, for example the file doesn't exist
        /// or <paramref name="file"/> is a directory.
        /// </exception>
        /// <exception cref="FileStatisticsException">
        /// If processing fails.
        /// </exception>
        public void ProcessFile(FileInfo file)
        {
            // cannot be null
            if (file == null)
            {
                throw new ArgumentNullException("file");
            }

            // throw IOException if file doesn't exist
            if (!(file.Exists))
            {
                throw new IOException(string.Format(
                    "File {0} doesn't exist", file.FullName));
            }

            try
            {
                // iterate processor
                foreach (IFileProcessor processor in this.processors)
                {
                    // is this processor matched?
                    if (processor.FileMatcher.Match(file))
                    {
                        // process it
                        StatsResult result = processor.Process(file);

                        // aggregate the result
                        Reporting.Aggregate(processor, file, result);

                        // we're done if matchall is false
                        if (!IsMatchAll)
                        {
                            break;
                        }
                    }
                }
            }
            catch (IOException)
            {
                // just re-throw
                throw;
            }
            catch (FileStatisticsException)
            {
                // just re-throw
                throw;
            }
            catch (Exception e)
            {
                // encapsulate all others in FileStatisticsException
                throw new FileStatisticsException(string.Format(
                    "Error processing file {0}", file.FullName), e);
            }
        }

        /// <summary>
        /// Gets the file processor with given alias.
        /// </summary>
        ///
        /// <remarks>
        /// If no such processor exists, <b>null</b> is returned.
        /// </remarks>
        ///
        /// <param name="alias">
        /// The alias of the processor or <b>null</b> if no processor with the specified alias.
        /// </param>
        ///
        /// <value>
        /// The <see cref="IFileProcessor"/> with the specified alias.
        /// </value>
        ///
        /// <exception cref="ArgumentNullException">
        /// <paramref name="alias"/> is a null reference.
        /// </exception>
        /// <exception cref="ArgumentException">
        /// <paramref name="alias"/> is empty after trimmed.
        /// </exception>
        public IFileProcessor this[string alias]
        {
            get
            {
                // cannot be null nor empty
                if (alias == null)
                {
                    throw new ArgumentNullException("alias");
                }
                if (alias.Trim().Length == 0)
                {
                    throw new ArgumentException("'alias' cannot be empty", "alias");
                }

                // iterate the list and return processor if the alias matches
                foreach (IFileProcessor processor in this.processors)
                {
                    if (processor.Alias == alias)
                    {
                        return processor;
                    }
                }

                // no match is found
                return null;
            }
        }

        /// <summary>
        /// Loads configured processors.
        /// </summary>
        ///
        /// <remarks>
        /// Used by constructor to process configuration values and instantiate
        /// the file processors.
        /// </remarks>
        ///
        /// <param name="nameSpace">
        /// The namespace to load configuration values from.
        /// </param>
        ///
        /// <exception cref="ConfigurationException">
        /// Any error occurs loading the file processor.
        /// </exception>
        private void LoadProcessors(string nameSpace)
        {
            try
            {
                ConfigManager cm = ConfigManager.GetInstance();

                // load processors names prefix
                string propName = ProcessorsPropertyKey + "." + namesPropertyKey;
                string[] procNames = cm.GetValues(nameSpace, propName);

                // required and cannot be empty
                if ((procNames == null) || (procNames.Length == 0))
                {
                    throw new ConfigurationException(string.Format(
                        "{0} is missing or empty", propName));
                }

                foreach (string procName in procNames)
                {
                    // instantiate each processor
                    string prefix = procName + ".";
                    IFileProcessor processor = (IFileProcessor) Helper.CreateInstance(
                        cm, nameSpace, null, prefix + ClassnamePropertyKey,
                        new string[] { nameSpace, prefix }, typeof(IFileProcessor));

                    // add it
                    AddFileProcessor(processor);
                }
            }
            catch (ConfigurationException)
            {
                // just re-throw
                throw;
            }
            catch (Exception e)
            {
                throw new ConfigurationException(string.Format(
                    "Error loading processor from configuration file namespace {0}",
                    nameSpace), e);
            }
        }

        /// <summary>
        /// Loads configured reporting.
        /// </summary>
        ///
        /// <remarks>
        /// Used by constructor to process configuration values and instantiate
        /// the statistic reporter.
        /// </remarks>
        ///
        /// <param name="nameSpace">
        /// The namespace to load configuration values from.
        /// </param>
        ///
        /// <exception cref="ConfigurationException">
        /// Any error occurs loading the reporting instance.
        /// </exception>
        private void LoadReporting(string nameSpace)
        {
            try
            {
                ConfigManager cm = ConfigManager.GetInstance();

                // load reporting class name
                string prefix = ReportingPropertyKey + ".";
                string propName = prefix + ClassnamePropertyKey;
                string reportName = cm.GetValue(nameSpace, propName);

                // required
                if (reportName == null)
                {
                    throw new ConfigurationException(string.Format(
                        "{0} is missing", propName));
                }

                // instantiate it
                IStatsReporting report = (IStatsReporting) Helper.CreateInstance(
                    cm, nameSpace, null, propName,
                    new string[] { nameSpace, prefix }, typeof(IStatsReporting));

                // save it
                this.Reporting = report;
            }
            catch (ConfigurationException)
            {
                // just re-throw
                throw;
            }
            catch (Exception e)
            {
                throw new ConfigurationException(string.Format(
                    "Error loading reporting instance from configuration file namespace {0}",
                    nameSpace), e);
            }
        }

        /// <summary>
        /// Loads matchall flag.
        /// </summary>
        ///
        /// <remarks>
        /// Used by constructor to process configuration values and
        /// check matchall flag.
        /// It's optional, default to false.
        /// Set to true only if 's "true", "yes" or "on" case-insensitively.
        /// </remarks>
        ///
        /// <param name="nameSpace">
        /// The namespace to load configuration values from.
        /// </param>
        ///
        /// <exception cref="ConfigurationException">
        /// Any error occurs loading the matchall flag.
        /// </exception>
        private void LoadMatchAll(string nameSpace)
        {
            try
            {
                ConfigManager cm = ConfigManager.GetInstance();

                // get matchall value
                string matchAllFlag = cm.GetValue(nameSpace, MatchAllPropertyKey);

                // optional
                if (matchAllFlag == null)
                {
                    return;
                }

                // set to true, if "true"/"yes"/"on"
                IsMatchAll = Helper.BoolValue(matchAllFlag);
            }
            catch (Exception e)
            {
                throw new ConfigurationException(string.Format(
                    "Error loading matchall flag from configuration file namespace {0}",
                    nameSpace), e);
            }
        }

        /// <summary>
        /// Return a list of <see cref="IArgumentHandler"/> instances
        /// to the command line interface in order to get configured.
        /// </summary>
        ///
        /// <remarks>
        /// If no <see cref="IArgumentHandler"/> exists in the hierarchy,
        /// an empty list is returned.
        /// An <see cref="IArgumentHandler"/> could possibly be implemented
        /// by <see cref="IFileProcessor"/>, <see cref="IFileMatcher"/>,
        /// <see cref="IStatsReporting"/>, or the <see cref="FileStatistics"/> itself.
        /// </remarks>
        ///
        /// <returns>
        /// A list containing all <see cref="IArgumentHandler"/> instances
        /// supported by this file statistic instance.
        /// </returns>
        internal IList GetAllArgumentHandlers()
        {
            ArrayList args = new ArrayList();

            // add all file processor and attached file matcher options
            // that implements IArgumentHandler
            foreach (IFileProcessor proc in AllFileProcessors)
            {
                if (proc is IArgumentHandler)
                {
                    args.Add(proc);
                }
                if (proc.FileMatcher is IArgumentHandler)
                {
                    args.Add(proc.FileMatcher);
                }
            }
            // add IStatsReporting if it implements IArgumentHandler
            if (Reporting is IArgumentHandler)
            {
                args.Add(Reporting);
            }
            // add myself if I implement IArgumentHandler
            if (this is IArgumentHandler)
            {
                args.Add(this);
            }

            return args;
        }
    }
}
