/*
 * IArgumentHandler.cs
 *
 * File Statistics
 * Version: 1.0
 *
 * Copyright (c) 2005, TopCoder, Inc. All rights reserved
 *
 * Authors: WishingBone, TCSDEVELOPER
 */

using System;
using System.Collections;

namespace TopCoder.File.Statistics
{
    /// <summary>
    /// Represents an entity that could potentially require arguments from the command line.
    /// </summary>
    ///
    /// <remarks>
    /// Currently the supported entities (that can potentially use arguments support)
    /// are <see cref="FileStatistics"/>, <see cref="IFileProcessor"/>, <see cref="IFileMatcher"/>
    /// and <see cref="IStatsReporting"/>.
    /// The procedure works in two phases.
    /// In the first step the handler proposes the options that it supports to the
    /// command line interface, where some simple validation is performed.
    /// In the second step the arguments are received at the handler and
    /// they can be further validated.
    /// </remarks>
    public interface IArgumentHandler
    {
        /// <summary>
        /// Proposes a list of <see cref="ArgumentOption"/> to the command line interface.
        /// </summary>
        ///
        /// <remarks>
        /// If the instance does not have anything to propose, an empty list can be returned.
        /// But to make more sense the entity does not need to implement this interface at the first place.
        /// </remarks>
        ///
        /// <returns>
        /// A <see cref="IList"/> of <see cref="ArgumentOption"/> instances.
        /// </returns>
        IList ProposeOptions();

        /// <summary>
        /// Receives the argument from the command line.
        /// </summary>
        ///
        /// <remarks>
        /// The key of the map is the switch name (String) and the value is the argument value (String).
        /// If a switch is not included in command line it should not show up in the map.
        /// If the switch is included in command line without any follow-up values an empty
        /// String ("") is provided as argument value.
        /// The interface only receives the arguments it has proposed.
        /// </remarks>
        ///
        /// <param name="arguments">
        /// The arguments map received.
        /// </param>
        ///
        /// <exception cref="ArgumentNullException">
        /// <paramref name="arguments"/> is a null reference.
        /// </exception>
        /// <exception cref="ArgumentException">
        /// If mapped value is not String (The content of the whole map is not validated).
        /// </exception>
        /// <exception cref="InvalidArgumentException">
        /// Some argument is not valid.
        /// </exception>
        void HandleArguments(IDictionary arguments);
    }
}
