/*
 * FileNameMatcher.cs
 *
 * File Statistics
 * Version: 1.0
 *
 * Copyright (c) 2005, TopCoder, Inc. All rights reserved
 *
 * Authors: WishingBone, TCSDEVELOPER
 */

using System;
using System.IO;

namespace TopCoder.File.Statistics.Matcher
{
    /// <summary>
    /// <see cref="FileNameMatcher"/> is a template class
    /// that handles filename oriented matching logic.
    /// </summary>
    ///
    /// <threadsafety static="true" instance="true"/>
    public abstract class FileNameMatcher : IFileMatcher
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="FileNameMatcher"/> instance.
        /// </summary>
        protected FileNameMatcher()
        {
        }

        /// <summary>
        /// Extracts the filename from the specified <paramref name="file"/>
        /// and delegates to the template method.
        /// </summary>
        ///
        /// <remarks>
        /// This method doesn't attempt to validate
        /// whether the specified file exists and valid.
        /// If a directory path is passed, this method will extract
        /// the last directory name.
        /// </remarks>
        ///
        /// <param name="file">
        /// The file which filename to match.
        /// </param>
        ///
        /// <returns>
        /// <b>true</b> if the filename match; otherwise, <b>false</b>.
        /// </returns>
        ///
        /// <exception cref="ArgumentNullException">
        /// <paramref name="file"/> is a null reference.
        /// </exception>
        /// <exception cref="FileMatchingException">
        /// An error occurs while trying to match the file name.
        /// </exception>
        public bool Match(FileInfo file)
        {
            // cannot be null
            if (file == null)
            {
                throw new ArgumentNullException("file");
            }

            try
            {
                // get the filename and call the template method
                return Match(file.Name);
            }
            catch (FileMatchingException)
            {
                // just re-throw
                throw;
            }
            catch (Exception e)
            {
                // encapsulate all other errors in FileMatchingException
                throw new FileMatchingException(string.Format(
                    "Error matching for filename {0}", file.FullName), e);
            }
        }

        /// <summary>
        /// The template method to match a filename.
        /// </summary>
        ///
        /// <param name="filename">
        /// The filename to match.
        /// </param>
        ///
        /// <returns>
        /// <b>true</b> if the filename match; otherwise, <b>false</b>.
        /// </returns>
        ///
        /// <exception cref="FileMatchingException">
        /// An error occurs while trying to match the file name.
        /// </exception>
        protected abstract bool Match(string filename);
    }
}
