/*
 * MimeMatcher.cs
 *
 * File Statistics
 * Version: 1.0
 *
 * Copyright (c) 2005, TopCoder, Inc. All rights reserved
 *
 * Authors: WishingBone, TCSDEVELOPER
 */

using System;
using System.Collections;
using TopCoder.Unix.MagicNumbers;
using TopCoder.Util.ConfigurationManager;

namespace TopCoder.File.Statistics.Matcher
{
    /// <summary>
    /// Matches file types with one or more MIMEs (as configured in Magic Numbers).
    /// </summary>
    ///
    /// <remarks>
    /// The file MIME is matched in a case-sensitive manner
    /// against a list of MIME names.
    /// Please note that the MIME names should match with those defined in
    /// the Magic Number component (case-sensitive).
    /// It is possible to instantiate this class using reflection
    /// and has its MIME list loaded from a configuration file.
    /// The constructor with nameSpace and prefix arguments is used for this purpose.
    /// There should be a multi-value property with the name <c>prefix + "mimes"</c>,
    /// which contains the MIME list.
    /// </remarks>
    ///
    /// <threadsafety static="true" instance="true"/>
    public class MimeMatcher : FileTypeMatcher
    {
        /// <summary>
        /// The configuration key that contains multiple file MIMEs to match.
        /// </summary>
        ///
        /// <remarks>
        /// The actual property key name is <c>prefix + "mimes"</c>.
        /// The matching is carried out in a case-sensitive manner.
        /// The property is required, and empty mimes is not allowed
        /// even if it is valid in the Magic Number component.
        /// </remarks>
        public static readonly string MimesPropertyKey = "mimes";

        /// <summary>
        /// A list of file mimes (string) to match.
        /// This list is initialized in constructor and is immutable.
        /// Developer notes. Although the design recommends IList, it's
        /// better to use IDictionary; because IList.Contains is O(n),
        /// while IDictionary.Contains can be O(1).
        /// It's currently implemented with a HashTable.
        /// </summary>
        private IDictionary mimes;

        /// <summary>
        /// Initializes a new instance of the <see cref="MimeMatcher"/>
        /// with the specified single file MIME.
        /// </summary>
        ///
        /// <param name="mime">
        /// A single file MIME to match.
        /// </param>
        ///
        /// <exception cref="ArgumentNullException">
        /// <paramref name="mime"/> is a null reference.
        /// </exception>
        /// <exception cref="ArgumentException">
        /// <paramref name="mime"/> is empty.
        /// </exception>
        public MimeMatcher(string mime)
        {
            // cannot be null nor empty
            if (mime == null)
            {
                throw new ArgumentNullException("mime");
            }
            if (mime.Trim().Length == 0)
            {
                throw new ArgumentException("'mime' cannot be empty", "mime");
            }

            // save to instance members
            this.mimes = new Hashtable();
            this.mimes.Add(mime, null);
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="MimeMatcher"/>
        /// with the specified list of file MIMEs.
        /// </summary>
        ///
        /// <param name="mimes">
        /// The list of file MIMEs (string) to match.
        /// </param>
        ///
        /// <exception cref="ArgumentNullException">
        /// <paramref name="mimes"/> is a null reference.
        /// </exception>
        /// <exception cref="ArgumentException">
        /// <paramref name="mimes"/> is empty, contains non-string instance,
        /// contains empty string (after trimmed), or contains duplicate element.
        /// </exception>
        public MimeMatcher(IList mimes)
        {
            // cannot be null
            if (mimes == null)
            {
                throw new ArgumentNullException("mimes");
            }
            // cannot be empty
            if (mimes.Count == 0)
            {
                throw new ArgumentException("'mimes' cannot be an empty list", "mimes");
            }
            // cannot contain non-string instance
            foreach (object obj in mimes)
            {
                if (!(obj is string))
                {
                    throw new ArgumentException(
                        "'mimes' contains a non-string instance", "mimes");
                }
                // cannot contain empty string
                if (((string) obj).Trim().Length == 0)
                {
                    throw new ArgumentException(
                        "'mimes' contains empty string", "mimes");
                }
            }

            // add list to hashtable (duplicate will throw ArgumentException)
            this.mimes = new Hashtable();
            Helper.AddListToDictionary(mimes, this.mimes);
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="MimeMatcher"/>
        /// which loads the file MIMEs list from the specified nameSpace
        /// and prefix of the configuration.
        /// </summary>
        ///
        /// <remarks>
        /// The actual property key name is <c>prefix + "mimes"</c>
        /// and multiple values are possible.
        /// </remarks>
        ///
        /// <param name="nameSpace">
        /// The namespace to load configuration values from.
        /// </param>
        /// <param name="prefix">
        /// The prefix of the property name to load configuration values from.
        /// </param>
        ///
        /// <exception cref="ArgumentNullException">
        /// Any of the argument is a null reference.
        /// </exception>
        /// <exception cref="ArgumentException">
        /// <paramref name="nameSpace"/> is empty.
        /// </exception>
        /// <exception cref="ConfigurationException">
        /// The property name cannot be found or
        /// it does not hold any values or any value is empty after trimmed
        /// or it contains duplicate values.
        /// </exception>
        public MimeMatcher(string nameSpace, string prefix)
        {
            // cannot be null
            if (nameSpace == null)
            {
                throw new ArgumentNullException("nameSpace");
            }
            if (prefix == null)
            {
                throw new ArgumentNullException("prefix");
            }
            // nameSpace cannot be empty
            if (nameSpace.Trim().Length == 0)
            {
                throw new ArgumentException("'nameSpace' cannot be empty", "nameSpace");
            }

            // the property name
            string propName = prefix + MimesPropertyKey;
            try
            {
                // load the multi-value property
                ConfigManager cm = ConfigManager.GetInstance();
                string[] mimes = cm.GetValues(nameSpace, propName);

                // required and cannot be empty
                if ((mimes == null) || (mimes.Length == 0))
                {
                    throw new ConfigurationException(string.Format(
                        "{0} cannot be found or is empty", propName));
                }

                // no element can be empty after trimmed
                foreach (string s in mimes)
                {
                    if (s.Trim().Length == 0)
                    {
                        throw new ConfigurationException(string.Format(
                            "{0} should not contain empty string", propName));
                    }
                }

                // add list to dictionary
                // (duplicate will throw ArgumentException, encapsulated to ConfigurationException)
                this.mimes = new Hashtable();
                Helper.AddListToDictionary(mimes, this.mimes);
            }
            catch (ConfigurationException)
            {
                // just re-throw
                throw;
            }
            catch (Exception e)
            {
                // encapsulate all others in ConfigurationException
                throw new ConfigurationException(string.Format(
                    "Error initializing from namespace {0}, property {1}", nameSpace, propName), e);
            }
        }

        /// <summary>
        /// Checks whether the file type MIME matches any of the MIMEs.
        /// </summary>
        ///
        /// <remarks>
        /// The MIME of the FileType will be compared in a case-sensitive
        /// manner with the registered MIMEs.
        /// </remarks>
        ///
        /// <param name="type">
        /// The FileType whose MIME to match.
        /// </param>
        ///
        /// <returns>
        /// <b>true</b> if the file type MIME match; otherwise, <b>false</b>
        /// </returns>
        protected override bool Match(FileType type)
        {
            // type.Mime could be null, just return false
            if (type.Mime == null)
            {
                return false;
            }

            // match against the list (O(1) operation)
            return mimes.Contains(type.Mime);
        }
    }
}
