/*
 * SimpleStateMachineWithAtLiteral.cs
 *
 * File Statistics
 * Version: 1.0
 *
 * Copyright (c) 2005, TopCoder, Inc. All rights reserved
 *
 * Author: TCSDEVELOPER
 */

using System;

namespace TopCoder.File.Statistics.Processor.LineCounter
{
    /// <summary>
    /// Implements a state machine that can process the source code.
    /// </summary>
    ///
    /// <remarks>
    /// This class is similar to the inner class <see cref="SimpleStateMachine"/>
    /// in <see cref="CStyleLineCounter"/>, but it supports C# at string literal.
    /// </remarks>
    ///
    /// <threadsafety static="false" instance="false"/>
    internal class SimpleStateMachineWithAtLiteral : IStateMachine
    {
        /// <summary>
        /// Represents the state of the machine.
        /// </summary>
        private enum State
        {
            /// <summary>
            /// Not in comment or literal.
            /// </summary>
            StateStart,

            /// <summary>
            /// After a single slash
            /// (Input on slash would enter line comment state;
            /// input on star would enter block state).
            /// </summary>
            StateSlash,

            /// <summary>
            /// After a single @ from Start.
            /// (A double quote would enter AtLiteral state)
            /// (Any other input would return to Start)
            /// </summary>
            StateAt,

            /// <summary>
            /// C# at style literal.
            /// After @"...
            /// A single quote would transfer to StateAtQuoteQuote.
            /// Any other character would preserve this state.
            /// </summary>
            StateAtQuote,

            /// <summary>
            /// A quote inside a C# at style literal.
            /// Another quote would transfer to StateAtQuote again;
            /// otherwise, go back to Start.
            /// </summary>
            StateAtQuoteQuote,

            /// <summary>
            /// A star in a block comment (Input of slash would enter start state).
            /// </summary>
            StateStar,

            /// <summary>
            /// Back slash in single quote literal (next character is escaped).
            /// </summary>
            StateSQBackslash,

            /// <summary>
            /// Back slash in double quote literal (next character is escaped).
            /// </summary>
            StateDQBackslash,

            /// <summary>
            /// Single quote literal.
            /// </summary>
            StateSingleQuote,

            /// <summary>
            /// Double quote literal.
            /// </summary>
            StateDoubleQuote,

            /// <summary>
            /// Block comment.
            /// </summary>
            StateBlockComment,

            /// <summary>
            /// Line comment.
            /// </summary>
            StateLineComment
        }

        /// <summary>
        /// The current state of this machine.
        /// </summary>
        private State state = State.StateStart;

        /// <summary>
        /// The current line count. Initialized to 0 and increments whenever necessary.
        /// </summary>
        private long lines = 0;

        /// <summary>
        /// Gets the code line count so far.
        /// </summary>
        ///
        /// <value>
        /// A long representing the code line count so far.
        /// </value>
        public long Lines
        {
            get
            {
                return lines;
            }
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="SimpleStateMachineWithAtLiteral"/> class.
        /// </summary>
        public SimpleStateMachineWithAtLiteral()
        {
        }

        /// <summary>
        /// Processes a line.
        /// </summary>
        ///
        /// <param name="line">
        /// The line to process (don't include the new line character).
        /// </param>
        ///
        /// <exception cref="ArgumentNullException">
        /// <paramref name="line"/> is a null reference.
        /// </exception>
        public void Process(string line)
        {
            // cannot be null
            if (line == null)
            {
                throw new ArgumentNullException("line");
            }
            // process each char
            foreach (char ch in line)
            {
                Process(ch);
            }
            // add the new line char
            Process('\n');
        }

        /// <summary>
        /// Processes a character and updates state accordingly.
        /// </summary>
        ///
        /// <param name="ch">
        /// The character to process.
        /// </param>
        private void Process(char ch)
        {
            switch (state)
            {
                // start state
                case State.StateStart:
                    switch (ch)
                    {
                        case ';':
                        case '{':
                            lines++;
                            break;
                        case '/':
                            state = State.StateSlash;
                            break;
                        case '\'':
                            state = State.StateSingleQuote;
                            break;
                        case '"':
                            state = State.StateDoubleQuote;
                            break;
                        case '@':
                            state = State.StateAt;
                            break;
                    }
                    break;

                // After an @
                case State.StateAt:
                    switch (ch)
                    {
                        case '"':
                            state = State.StateAtQuote;
                            break;
                        default:
                            state = State.StateStart;
                            break;
                    }
                    break;

                // After an @"
                case State.StateAtQuote:
                    switch (ch)
                    {
                        case '"':
                            state = State.StateAtQuoteQuote;
                            break;
                    }
                    break;

                // After an @" " (because it's possible to specify double quotes to mark 1 quote)
                case State.StateAtQuoteQuote:
                    switch (ch)
                    {
                        case '"':
                            // the user wants a single quote inside @".."".."
                            state = State.StateAtQuote;
                            break;
                        default:
                            state = State.StateStart;
                            // we need to push back and process this char
                            Process(ch);
                            break;
                    }
                    break;

                // slash state
                case State.StateSlash:
                    switch (ch)
                    {
                        case '*':
                            state = State.StateBlockComment;
                            break;
                        case '/':
                            state = State.StateLineComment;
                            break;
                        default:
                            state = State.StateStart;
                            break;
                    }
                    break;

                // star state (star in block comment)
                case State.StateStar:
                    switch (ch)
                    {
                        case '/':
                            state = State.StateStart;
                            break;
                        case '*':
                            state = State.StateStar;
                            break;
                        default:
                            state = State.StateBlockComment;
                            break;
                    }
                    break;

                // SQ backslash state (backslash in single quote)
                case State.StateSQBackslash:
                    state = State.StateSingleQuote;
                    break;

                // DQ backslash state (backslash in double quote)
                case State.StateDQBackslash:
                    state = State.StateDoubleQuote;
                    break;

                // Single quote state
                case State.StateSingleQuote:
                    switch (ch)
                    {
                        case '\\':
                            state = State.StateSQBackslash;
                            break;
                        case '\'':
                            state = State.StateStart;
                            break;
                    }
                    break;

                // Double quote state
                case State.StateDoubleQuote:
                    switch (ch)
                    {
                        case '\\':
                            state = State.StateDQBackslash;
                            break;
                        case '"':
                            state = State.StateStart;
                            break;
                    }
                    break;

                // line comment state
                case State.StateLineComment:
                    switch (ch)
                    {
                        case '\n':
                            state = State.StateStart;
                            break;
                    }
                    break;

                // line comment state
                case State.StateBlockComment:
                    switch (ch)
                    {
                        case '*':
                            state = State.StateStar;
                            break;
                    }
                    break;
            }
        }
    }
}
