/*
Copyright (c)2005, TopCoder, Inc. All rights reserved
Author Cucu
*/
using NUnit.Framework;
using TopCoder.File.Statistics;
using System.IO;

using System.Text.RegularExpressions;
using System.Collections;
using TopCoder.Util.ConfigurationManager;
using TopCoder.File.Statistics.AccuracyTests;
using TopCoder.File.Statistics.Reporting;

namespace TopCoder.File.Statistics.AccuracyTests
{
    /// <summary>
    /// Accuracy tests for FileStatistics
    /// </summary>
    [TestFixture]
    public class FileStatisticsAccuracyTest
    {       

        /// <summary>
        /// Object to be tested
        /// </summary>
        private FileStatistics fs;

        /// <summary>
        /// Set up for each test.
        /// </summary>
        [SetUp]
        public void SetUp()
        {
            TestHelper.LoadConfig(TestHelper.FILE_STATISTICS_CONFIG);
            fs = new FileStatistics(TestHelper.FILE_STATISTICS_NAMESPACE);
        }

        /// <summary>
        /// Tear down for each test.
        /// </summary>
        [TearDown]
        public void Teardown()
        {
            TestHelper.UnloadConfig(TestHelper.FILE_STATISTICS_CONFIG);
        }

        /// <summary>
        /// Test that the configuration is correctly loaded from configuration manager
        /// </summary>
        [Test]
        public void TestConstructorFromConfig()
        {
            Assert.IsTrue(fs.IsMatchAll, "MatchAll should be true");
            Assert.IsTrue(fs.Reporting is XmlStatsReporting, "Reporting should be an instance of XmlStatsReporting");
            Assert.AreEqual("Text", ((IFileProcessor) fs.AllFileProcessors[0]).Alias, "bad File Processor loaded");
            Assert.AreEqual("Source", ((IFileProcessor) fs.AllFileProcessors[1]).Alias, "bad File Processor loaded");
        }

        /// <summary>
        /// Test the indexer that retrieves a file processor from its alias
        /// </summary>
        [Test]
        public void TestIndexer()
        {
            Assert.AreEqual("Text", fs["Text"].Alias, "bad File Processor retrieved");
            Assert.IsNull(fs["Inexistent"], "null expeceted for non existant alias");
        }

        /// <summary>
        /// Test RemoveFileProcessor method
        /// </summary>
        [Test]
        public void TestRemoveFileProcessor()
        {
            fs.RemoveFileProcessor("Text");
            Assert.IsNull(fs["Text"], "File Processor not deleted");
        }

        /// <summary>
        /// Test the ClearAllFileProcessors method
        /// </summary>
        [Test]
        public void TestClear() 
        {
            fs.ClearAllFileProcessors();
            Assert.AreEqual(0, fs.AllFileProcessors.Count, "Clear is not removing all the elements");
        }

        /// <summary>
        /// Test the ProcessFile method with a file that should be processed by both processors
        /// </summary>
        [Test]
        public void TestProcessFile1() 
        {
            fs.ProcessFile(new FileInfo(TestHelper.CS_FILE));
            string report = fs.Reporting.GenerateReport();

            // Save to a file for visual inspection
            TestHelper.SaveToFile(@"FileStatisticsResults\ProcessFile1.xml", report);        

            Assert.IsTrue(report.IndexOf("<line>") >= 0, "Report doesn't seem to be right");
            Assert.IsTrue(report.IndexOf("Source") >= 0, "Report doesn't seem to be right");
            Assert.IsTrue(report.IndexOf("Text") >= 0, "Report doesn't seem to be right");            
        }

        /// <summary>
        /// Test the ProcessFile method with a file that should be processed just by the Text Processor
        /// </summary>
        [Test]
        public void TestProcessFile2() 
        {
            fs.ProcessFile(new FileInfo(TestHelper.BIG_TXT_FILE));
            string report = fs.Reporting.GenerateReport();

            // Save to a file for visual inspection
            TestHelper.SaveToFile(@"FileStatisticsResults\ProcessFile2.xml", report);                
            
            Assert.IsTrue(report.IndexOf("<line>") >= 0, "Report doesn't seem to be right");
            Assert.IsTrue(report.IndexOf("Text") >= 0, "Report doesn't seem to be right");
            Assert.IsTrue(report.IndexOf("Source") < 0, "Report doesn't seem to be right");
            
        }

        /// <summary>
        /// Test the ProcessFile method with a file that matches both processors, but setting isMatchAll to false
        /// so that just the Text processor must process it
        /// </summary>
        [Test]
        public void TestProcessFile3() 
        {
            fs.IsMatchAll = false;

            fs.ProcessFile(new FileInfo(TestHelper.CS_FILE));
            string report = fs.Reporting.GenerateReport();

            // Save to a file for visual inspection
            TestHelper.SaveToFile(@"FileStatisticsResults\ProcessFile3.xml", report);        

            Assert.IsTrue(report.IndexOf("<line>") >= 0, "Report doesn't seem to be right");
            Assert.IsTrue(report.IndexOf("Source") < 0, "Report doesn't seem to be right");
            Assert.IsTrue(report.IndexOf("Text") >= 0, "Report doesn't seem to be right");                       
        }

        /// <summary>
        /// Test the ProcessFile method with 4 files: one that doesn't match anything, one that just matches the text
        /// processor, one that just matches the source processor and the last that matches both
        /// </summary>
        [Test]
        public void TestProcessFile4() 
        {
            fs.ProcessFile(new FileInfo(TestHelper.CS_FILE));
            fs.ProcessFile(new FileInfo(TestHelper.CPP_FILE));
            fs.ProcessFile(new FileInfo(TestHelper.DLL_FILE));
            fs.ProcessFile(new FileInfo(TestHelper.TXT_FILE));
            string report = fs.Reporting.GenerateReport();

            // Save to a file for visual inspection
            TestHelper.SaveToFile(@"FileStatisticsResults\ProcessFile4.xml", report);                

            Assert.IsTrue(report.IndexOf("<line>") >= 0, "Report doesn't seem to be right");
            Assert.IsTrue(report.IndexOf("Source") >= 0, "Report doesn't seem to be right");
            Assert.IsTrue(report.IndexOf("Text") >= 0, "Report doesn't seem to be right");
            
        }

    }    
}