/*
Copyright (c)2005, TopCoder, Inc. All rights reserved
Author Cucu
*/

using NUnit.Framework;
using TopCoder.File.Statistics;
using System.IO;
using System.Collections;
using TopCoder.Util.ConfigurationManager;
using System.Text.RegularExpressions;
using TopCoder.File.Statistics.AccuracyTests;
using TopCoder.File.Statistics.Matcher;

namespace TopCoder.File.Statistics.AccuracyTests.Matcher
{
    /// <summary>
    /// Accuracy tests for RegexMatcher.
    /// </summary>
    [TestFixture]
    public class RegexMatcherAccuracyTest
    {              
        /// <summary>
        /// Set up for each test.
        /// </summary>
        [SetUp]
        public void SetUp()
        {
            TestHelper.LoadConfig(TestHelper.MATCHERS_CONFIG);
        }

        /// <summary>
        /// Tear down for each test.
        /// </summary>
        [TearDown]
        public void Teardown()
        {
            TestHelper.UnloadConfig(TestHelper.MATCHERS_CONFIG);
        }

        /// <summary>
        /// Test the Match method against a single pattern.
        /// </summary>
        [Test]
        public void MatchWithStringConstructor()
        {
            IFileMatcher matcher = new RegexMatcher(new Regex(@"^file[0-9][0-9]"));
            
            Assert.IsTrue(matcher.Match(new FileInfo(@"file99")), @"file99 should match with the regex");
            Assert.IsTrue(matcher.Match(new FileInfo(@"file12.txt")), @"file12.txt should match with the regex");
            Assert.IsTrue(matcher.Match(new FileInfo(@"c:\doc\file34.cpp")), @"c:\doc\file34.cpp should match with the regex");
            Assert.IsTrue(matcher.Match(new FileInfo(@"c:\file00.cpp.bak")), @"c:\dfile00.cpp.bak should match with the regex");

            Assert.IsFalse(matcher.Match(new FileInfo(@"myfile00.cpp")), @"myFile00.cpp should NOT match with the regex");
            Assert.IsFalse(matcher.Match(new FileInfo(@"FiLe00.cpp")), @"myFile00.cpp should NOT match with the regex");
            Assert.IsFalse(matcher.Match(new FileInfo(@"FiLe00.cpp\efile23.abc")), @"FiLe00.cpp\file23.abc should NOT match with the regex");
        }

        /// <summary>
        /// Test that the list constructor makes a copy of the list
        /// </summary>
        [Test]
        public void TestListConstructor() 
        {
            IList patterns  = new ArrayList();
            patterns.Add(new Regex("x"));
            
            IFileMatcher matcher = new RegexMatcher(patterns);
            
            // this shouldn't affect the matcher
            patterns.Add(new Regex("a"));
            
            Assert.IsFalse(matcher.Match(new FileInfo(@"abc")), @"constructor is not making a copy");
        }

        
        /// <summary>
        /// Test the Match method against many patterns, using the constructor that takes a list of Regex.
        /// </summary>
        [Test]
        public void MatchWithListConstructor()
        {
            IList patterns = new ArrayList();
            patterns.Add(new Regex(@"\.mp[\d]$"));
            patterns.Add(new Regex(@"TopCoder"));
            patterns.Add(new Regex(@"hello"));
            
            IFileMatcher matcher = new RegexMatcher(patterns);

            checkPatterns(matcher);
        }

        /// <summary>
        /// Test the Match method against many extensions, using the constructor that reads from the configuration file.
        /// </summary>
        [Test]
        public void MatchWitConfigManagerConstructor()
        {
            IFileMatcher matcher = new RegexMatcher(TestHelper.FILE_STATISTICS_NAMESPACE, "regex.matcher.");
            checkPatterns(matcher);
        }

        /// <summary>
        /// Check that the matcher passed is matching well with the extensions: cpp, cs, java and pas.
        /// </summary>
        /// <param name="matcher">the matcher to check.</param>
        private void checkPatterns(IFileMatcher matcher) {
            Assert.IsTrue(matcher.Match(new FileInfo(@"music.mp3")), @"music.mp3 should match with the patterns");
            Assert.IsTrue(matcher.Match(new FileInfo(@"e:\music.mp4")), @"e:\music.mp4 should match with the patterns");
            Assert.IsTrue(matcher.Match(new FileInfo(@"TopCoder.txt")), @"TopCoder.txt should match with the patterns");
            Assert.IsTrue(matcher.Match(new FileInfo(@"say_hello_world.txt")), @"say_hello_world.txt should match with the patterns");

            Assert.IsFalse(matcher.Match(new FileInfo(@"music.mp3.bak")), @"music.mp3.bak should NOT match with the patterns");
            Assert.IsFalse(matcher.Match(new FileInfo(@"e:\music.mpx")), @"e:\music.mpx should NOT match with the patterns");
            Assert.IsFalse(matcher.Match(new FileInfo(@"topcoder.txt")), @"topcoder.txt should match with the patterns");

        }

         }    
}