/*
Copyright (c)2005, TopCoder, Inc. All rights reserved
Author Cucu
*/

using NUnit.Framework;

using TopCoder.File.Statistics.Reporting;
using TopCoder.File.Statistics;

namespace TopCoder.File.Statistics.AccuracyTests.Reporting
{
    /// <summary>
    /// Accuracy tests for StatsCollection
    /// </summary>
    [TestFixture]
    public class StatsCollectionAccuracyTest
    {       
        /// <summary>
        /// StatsResult used for testing.  Values will be set in set up.
        /// </summary>
        private StatsResult sr1 = null;

        /// <summary>
        /// StatsResult used for testing.  Values will be set in set up.
        /// </summary>
        private StatsResult sr2 = null;
        
        /// <summary>
        /// Metric used for testing.  
        /// </summary>
        private Metric m = new Metric("test","test","test");

        /// <summary>
        /// The StatsCollection object to be tested.
        /// </summary>
        private StatsCollection sc = null;

        /// <summary>
        /// Set up for each test.
        /// </summary>
        [SetUp]
        public void SetUp() 
        {
            sr1 = new StatsResult();
            sr2 = new StatsResult();
            sc = new StatsCollection();

            // set metric values for the results.
            sr1[m] = 1;
            sr2[m] = 2;
        }
        
        /// <summary>
        /// Test that the constructor creates an empty set.
        /// </summary>
        [Test]
        public void TestConstructor() 
        {
            Assert.AreEqual(0, sc.AllKeys.Count, "StatsCollection should be created without any StatsResult");
        }

        /// <summary>
        /// Test the add method to verify that it adds results and that it makes a copy of the inserted object.
        /// </summary>
        [Test]
        public void TestAdd() 
        {
            sc.Add("sr1", sr1);
            sc.Add("sr2", sr2);
            
            Assert.AreEqual(2, sc.AllKeys.Count, "StatsCollection should have 2 elements");
            Assert.AreEqual(sr1[m], sc["sr1"][m], "bad balue retrieved for sr1");
            Assert.AreEqual(sr2[m], sc["sr2"][m], "bad balue retrieved for sr2");

            sr1.Aggregate(sr2);
            Assert.AreEqual(1, sc["sr1"][m], "Add method is not making a copy");
        }

        /// <summary>
        /// Test that the add method aggregates results
        /// </summary>
        [Test]
        public void TestAddAggregate() 
        {
            sc.Add("result1", sr1);
            sc.Add("result1", sr1);
            sc.Add("result1", sr1);
            sc.Add("result1", sr1);
            sc.Add("result1", sr1);
           
            sc.Add("result2", sr2);
            
            Assert.AreEqual(2, sc.AllKeys.Count, "StatsCollection should have 2 elements");
            Assert.AreEqual(sr1[m] * 5 , sc["result1"][m], "bad balue retrieved for result1");
            Assert.AreEqual(sr2[m], sc["result2"][m], "bad balue retrieved for result2");
        }

        /// <summary>
        /// Test that the AllKey property returns a sorted case insensitive list of keys.
        /// </summary>
        [Test]
        public void TestAllKey()
        {
            sc.Add("a", sr1);
            sc.Add("B", sr2);
            
            Assert.AreEqual("a", sc.AllKeys [0], "bad key value");
            Assert.AreEqual("B", sc.AllKeys [1], "bad key value");
        }

        /// <summary>
        /// Test that the indexer retrieves the right value and that it makes a copy.
        /// </summary>
        [Test]
        public void TestIndexer() 
        {
            sc.Add("sr1", sr1);
            sc.Add("sr2", sr2);

            Assert.AreEqual(sr1[m], sc["sr1"][m], "bad value retrieved");
            Assert.IsNull(sc["inexistent"], "indexer must return null if the key was not found");

            // Test that the indexer makes a copy
            StatsResult srCopy = sc["sr1"];
            srCopy.Aggregate(sr2);
            Assert.AreEqual(1, sc["sr1"][m], "The indexer is not returning a copy of the object");

        }
        /// <summary>
        /// Test the clear method
        /// </summary>
        [Test]
        public void TestClear() {           
            sc.Add("sr1", sr1);

            sc.Clear();
            Assert.AreEqual(0, sc.AllKeys.Count, "Clear should erase all the elements");
        }

    }    
}