///
/// $(#)MimeMatcherTests.java		Sep 6, 2005
/// 
/// Copyright (c) 2005, TopCoder Inc. All rights reserved.
/// </summary>

using System;
using System.IO;
using System.Collections;
using System.Text.RegularExpressions;

using NUnit.Framework;

using TopCoder.File.Statistics.Matcher;
using TopCoder.Util.ConfigurationManager;

namespace TopCoder.File.Statistics.FailureTests.Matcher
{
	/// <summary>
    /// This test fixture contains the failure tests for <see cref="RegexMatcher"/> class.
    ///
    /// <author>MPhk</author>
    /// <version>1.0</version>
    /// <copyright>Copyright (C) 2005 TopCoder Inc., All Rights Reserved.</copyright>
    /// </summary>
    [TestFixture]
	public class RegexMatcherTests {
	
	    /// <summary>
	    /// The namespace for this test.
	    /// </summary>
	    private const string NAMESPACE = "TopCoder.File.Statistics.Matcher.FailureTests";
	
	    /// <summary>
	    /// The config file for this test.
	    /// </summary>
	    private const string CONFIG_FILE = "../../test_files/failuretests/matcher/config.xml";
	
	    /// <summary>
	    /// An RegexMatcher instance to test.
	    /// </summary>
	    private RegexMatcher matcher = null;
	
	    /// <summary>
	    /// Load the configuration before testing.
	    /// </summary>
	    [SetUp]
	    protected void SetUp() {
	        // clear the namespace first
	        ConfigManager configManager = ConfigManager.GetInstance();
	        configManager.RemoveFile(CONFIG_FILE);
	        // load the configuration
	        configManager.LoadFile(CONFIG_FILE);
	        
	        matcher = new RegexMatcher(new Regex("^.*\\.txt$"));
	    }
	
	    /// <summary>
	    /// Remove the configuration after testing.
	    /// </summary>
	    [TearDown]
	    protected void TearDown() {
	        ConfigManager configManager = ConfigManager.GetInstance();
	        configManager.RemoveFile(CONFIG_FILE);
	    }
	
	    /// <summary>
	    /// Checks whether the constructor RegexMatcher(Regex) properly throws
	    /// ArgumentNullException for null argument.
	    /// </summary>
	    [Test, ExpectedException(typeof(ArgumentNullException))]
	    public void TestRegexMatcher1() {
            new RegexMatcher((Regex)null);
	    }
	
	    /// <summary>
	    /// Checks whether the constructor RegexMatcher(string, string) properly throws
	    /// ArgumentNullException for null argument.
	    /// </summary>
	    [Test, ExpectedException(typeof(ArgumentNullException))]
	    public void TestRegexMatcher2() {
            new RegexMatcher(null, "prefix");
	    }
	    
	    /// <summary>
	    /// Checks whether the constructor RegexMatcher(string, string) properly throws
	    /// ArgumentException for empty argument.
	    /// </summary>
	    [Test, ExpectedException(typeof(ArgumentException))]
	    public void TestRegexMatcher3() {
            new RegexMatcher("", "prefix");
	    }
	    
	    /// <summary>
	    /// Checks whether the constructor RegexMatcher(string, string) properly throws
	    /// ArgumentNullException for null argument.
	    /// </summary>
	    [Test, ExpectedException(typeof(ArgumentNullException))]
	    public void TestRegexMatcher4() {
            new RegexMatcher("namespace", null);
	    }
	
	    /// <summary>
	    /// Checks whether the constructor RegexMatcher(Property) properly throws
	    /// Configuration for invalid property - no values in "extensions" sub-property.
	    /// </summary>
	    [Test, ExpectedException(typeof(ConfigurationException))]
	    public void TestRegexMatcher5() {
            new RegexMatcher(NAMESPACE, "regex_invalid1");
	    }
	
	    /// <summary>
	    /// Checks whether the constructor RegexMatcher(Property) properly throws
	    /// Configuration for invalid property - no "extensions" sub-property.
	    /// </summary>
	    [Test, ExpectedException(typeof(ConfigurationException))]
	    public void TestRegexMatcher6() {
            new RegexMatcher(NAMESPACE, "regex_invalid2");
	    }
	
	    /// <summary>
	    /// Checks whether the constructor RegexMatcher(Property) properly throws
	    /// Configuration for invalid property - wrong pattern.
	    /// </summary>
	    [Test, ExpectedException(typeof(ConfigurationException))]
	    public void TestRegexMatcher7() {
            new RegexMatcher(NAMESPACE, "regex_invalid3");
	    }
	
	    /// <summary>
	    /// Checks whether the constructor RegexMatcher(List) properly throws
	    /// ArgumentNullException for null argument.
	    /// </summary>
	    [Test, ExpectedException(typeof(ArgumentNullException))]
	    public void TestRegexMatcher8() {
            new RegexMatcher((IList)null);
	    }
	
	    /// <summary>
	    /// Checks whether the constructor RegexMatcher(List) properly throws
	    /// ArgumentException for empty List argument.
	    /// </summary>
	    [Test, ExpectedException(typeof(ArgumentException))]
	    public void TestRegexMatcher9() {
            new RegexMatcher(new ArrayList());
	    }
	    
	    /// <summary>
	    /// Checks whether the constructor RegexMatcher(List) properly throws
	    /// ArgumentException for List argument containing nulls.
	    /// </summary>
	    [Test, ExpectedException(typeof(ArgumentException))]
	    public void TestRegexMatcher10() {
	        IList list = new ArrayList();
	        list.Add(new Regex("^.*\\.txt$"));
	        list.Add(null);

	        new RegexMatcher(list);
	    }
	    
	    /// <summary>
	    /// Checks whether the constructor RegexMatcher(List) properly throws
	    /// ArgumentException for List argument containing non String elements.
	    /// </summary>
	    [Test, ExpectedException(typeof(ArgumentException))]
	    public void TestRegexMatcher11() {
	        IList list = new ArrayList();
	        list.Add(new Regex("^.*\\.txt$"));
	        list.Add(new Object());

	        new RegexMatcher(list);
	    }
	    
	    /// <summary>
	    /// Tests the match method with null argument. ArgumentNullException should be thrown.
	    /// </summary>
	    [Test, ExpectedException(typeof(ArgumentNullException))]
	    public void TestMatch() {
	    	matcher.Match((FileInfo) null);
	    }
    }
}
