/*
 * TestStatsCollection.cs
 *
 * File Statistics
 * Version: 1.0
 *
 * Copyright (c) 2005, TopCoder, Inc. All rights reserved
 *
 * Author: TCSDEVELOPER
 */

using System;
using System.Collections;
using NUnit.Framework;

namespace TopCoder.File.Statistics.Reporting.Tests
{
    /// <summary>
    /// Unit tests for the <see cref="StatsCollection"/> class.
    /// </summary>
    [TestFixture]
    public class TestStatsCollection
    {
        /// <summary>
        /// A sample Metric name.
        /// </summary>
        private const string name = "Word count";

        /// <summary>
        /// A sample Metric display name.
        /// </summary>
        private const string displayName = "wcount";

        /// <summary>
        /// A sample Metric description.
        /// </summary>
        private const string description = "Count the number of words";

        /// <summary>
        /// An initial key name.
        /// </summary>
        private const string statsKey = "Stats Key";

        /// <summary>
        /// A sample initial metric to be stored in StatsResult.
        /// Initialized in SetUp().
        /// </summary>
        private Metric metric;

        /// <summary>
        /// The <see cref="StatsCollection"/> instance to test.
        /// Initialized in SetUp().
        /// </summary>
        private StatsCollection stats;

        /// <summary>
        /// Initializes the <see cref="StatsCollection"/> instance to test.
        /// </summary>
        [SetUp]
        public void SetUp()
        {
            stats = new StatsCollection();

            // create a StatsResult containing a metric with an initial value of 10
            metric = new Metric(name, displayName, description);
            StatsResult result = new StatsResult();
            result[metric] = 10;

            stats.Add(statsKey, result);
        }

        /// <summary>
        /// Tests AllKeys and this[key] property getter.
        /// </summary>
        [Test]
        public void TestAllKeys()
        {
            // assert 1 stats
            IList keys = stats.AllKeys;
            Assert.AreEqual(1, keys.Count, "1 key is stored");
            Assert.AreEqual(statsKey, keys[0], "The key name is incorrect");

            // assert the StatsResult
            StatsResult result = stats[(string) keys[0]];
            // 1 metric is stored
            IList metrics = result.Metrics;
            Assert.AreEqual(1, metrics.Count, "1 metric is stored");

            // assert the metric
            Metric m = metrics[0] as Metric;
            Assert.AreEqual(metric, m, "The Metric is incorrect");
            Assert.AreEqual(10, result[m], "The metric value should be 10");
        }

        /// <summary>
        /// Tests Add by adding two stats result with different keys.
        /// </summary>
        [Test]
        public void TestAdd()
        {
            // create a StatsResult containing a metric with an initial value of 7
            Metric newMetric = new Metric(name, displayName, description);
            StatsResult newResult = new StatsResult();
            newResult[newMetric] = 7;

            // add it with a key of "zzz"
            stats.Add("zzz", newResult);
            // add it again with a key of "abc"
            stats.Add("abc", newResult);

            // assert 3 stats (sorted case-insensitively)
            IList keys = stats.AllKeys;
            Assert.AreEqual(3, keys.Count, "3 keys are stored");
            Assert.AreEqual("abc", keys[0], "The 1st key name is incorrect");
            Assert.AreEqual(statsKey, keys[1], "The 2nd key name is incorrect");
            Assert.AreEqual("zzz", keys[2], "The 3rd key name is incorrect");

            // assert "abc" StatsResult and its 1 metric
            StatsResult result = stats["abc"];
            Assert.AreEqual(1, result.Metrics.Count, "1 metric is stored in abc");
            Assert.AreEqual(7, result[newMetric], "The metric value is 7");

            // assert "Stats Key" StatsResult and its 1 metric
            result = stats[statsKey];
            Assert.AreEqual(1, result.Metrics.Count, "1 metric is stored in Stats Key");
            Assert.AreEqual(10, result[newMetric], "The metric value is 10");

            // assert "abc" StatsResult and its 1 metric
            result = stats["zzz"];
            Assert.AreEqual(1, result.Metrics.Count, "1 metric is stored in zzz");
            Assert.AreEqual(7, result[newMetric], "The metric value is 7");
        }

        /// <summary>
        /// Tests Add by adding an empty stats result with the same key name.
        /// </summary>
        [Test]
        public void TestAddEmpty()
        {
            // create an empty StatsResult
            StatsResult newResult = new StatsResult();

            // add it with the same key name
            stats.Add(statsKey, newResult);

            // assert 1 stats
            IList keys = stats.AllKeys;
            Assert.AreEqual(1, keys.Count, "1 key is stored");
            Assert.AreEqual(statsKey, keys[0], "The key name is incorrect");

            // assert "Stats Key" StatsResult and its 1 metric
            StatsResult result = stats[statsKey];
            Assert.AreEqual(1, result.Metrics.Count, "1 metric is stored in Stats Key");
            Assert.AreEqual(10, result[metric], "The metric value is 10");
        }

        /// <summary>
        /// Tests Add aggregate StatsResult by adding to the same key.
        /// </summary>
        [Test]
        public void TestAddAggregate()
        {
            // create a StatsResult containing two metrics
            // the 1st metric is the same as existing metric
            // the 2nd metric is a new metric
            Metric newMetric = new Metric(name + "1", displayName, description);
            StatsResult newResult = new StatsResult();
            newResult[metric] = 2;      // the existing metric name
            newResult[newMetric] = 3;   // the new metric name

            // add it with existing key
            stats.Add(statsKey, newResult);

            // assert 1 stat
            IList keys = stats.AllKeys;
            Assert.AreEqual(1, keys.Count, "1 key is stored");
            Assert.AreEqual(statsKey, keys[0], "The key name is incorrect");

            // assert "Stats Key" StatsResult and its now 2 metrics
            StatsResult result = stats[statsKey];
            IList metrics = result.Metrics;
            Assert.AreEqual(2, metrics.Count, "2 metrics are stored in Stats Key");

            // determine which one is the first because there's no guarantee on the order
            int first = (((Metric) metrics[0]).Name == name) ? 0 : 1;

            // assert 1st Metric is now 10 + 2
            Assert.AreEqual(12, result[metric], "The metric value is 12");

            // assert 2nd Metric is 3
            Assert.AreEqual(3, result[newMetric], "The metric value is 3");
        }

        /// <summary>
        /// Tests Add throws ArgumentNullException
        /// given null key.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestAddNullKey()
        {
            stats.Add(null, new StatsResult());
        }

        /// <summary>
        /// Tests Add throws ArgumentNullException
        /// given null StatsResult.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestAddNullStatsResult()
        {
            stats.Add(statsKey, null);
        }

        /// <summary>
        /// Tests Add throws ArgumentException
        /// given empty key.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestAddEmptyKey()
        {
            stats.Add(" ", new StatsResult());
        }

        /// <summary>
        /// Tests Clear().
        /// </summary>
        [Test]
        public void TestClear()
        {
            stats.Clear();

            // no stats now
            Assert.AreEqual(0, stats.AllKeys.Count, "All keys cleared");
        }

        /// <summary>
        /// Tests this[key] property getter returns null
        /// given unknown key.
        /// </summary>
        [Test]
        public void TestIndexerGetUnknown()
        {
            Assert.IsNull(stats["unknown"], "Unknown key name should return null");
        }

        /// <summary>
        /// Tests this[key] property getter throws ArgumentNullException
        /// given null key.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestIndexerGetNull()
        {
            StatsResult result = stats[null];
        }

        /// <summary>
        /// Tests this[key] property getter throws ArgumentException
        /// given empty key.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestIndexerGetEmpty()
        {
            StatsResult result = stats[" "];
        }
    }
}
