/*
 * TestCommandLine.cs
 *
 * File Statistics
 * Version: 1.0
 *
 * Copyright (c) 2005, TopCoder, Inc. All rights reserved
 *
 * Author: TCSDEVELOPER
 */

using System;
using System.IO;
using NUnit.Framework;
using TopCoder.File.Statistics.Tests;
using TopCoder.Util.ConfigurationManager;

namespace TopCoder.File.Statistics.Tests
{
    /// <summary>
    /// Unit tests for the <see cref="CommandLine"/> class.
    /// </summary>
    [TestFixture]
    public class TestCommandLine : TestHelper
    {
        /// <summary>
        /// Saves the original console stream.
        /// Initialized in SetUp() and used in TearDown().
        /// </summary>
        private TextWriter originalConsole;

        /// <summary>
        /// The new redirected console. Initialized in SetUp().
        /// Console output will go here.
        /// We're going to assert the console output using this TextWriter.
        /// </summary>
        private TextWriter newConsole;

        /// <summary>
        /// Redirects standard console output temporarily to a string writer.
        /// </summary>
        [SetUp]
        public void SetUp()
        {
            // clear config manager
            ConfigManager.GetInstance().Clear();

            // save original console first
            originalConsole = Console.Out;

            // redirect console output to a StringWriter
            newConsole = new StringWriter();
            Console.SetOut(newConsole);
        }

        /// <summary>
        /// Restores standard console output.
        /// </summary>
        [TearDown]
        public void TearDown()
        {
            // restore console
            Console.SetOut(originalConsole);

            // clear config manager
            ConfigManager.GetInstance().Clear();
        }

        /// <summary>
        /// Tests CommandLine console output accuracy.
        /// </summary>
        [Test]
        public void TestCommandLineAccuracyConsole()
        {
            // run the command line
            CommandLine.Main(new string[]
                {
                    @"-c:..\..\test_files\FileStatistics_CommandLine.xml",
                    @"-b:..\..\test_files\test",
                    @"-r",
                    @"..\..\test_files\test"
                });

            // assert the output
            string actual = newConsole.ToString();
            string expected = ReadContent(@"..\..\test_files\ExtendedStatistics_SampleReport.xml", false);

            Assert.AreEqual(expected, actual, "The console output is wrong");
        }

        /// <summary>
        /// Tests CommandLine file output accuracy.
        /// </summary>
        [Test]
        public void TestCommandLineAccuracyFile()
        {
            string fileOutput = @"..\..\test_files\TempOutput.xml";

            try
            {
                // run the command line
                CommandLine.Main(new string[]
                {
                    @"-c:..\..\test_files\FileStatistics_CommandLine.xml",
                    @"-b:..\..\test_files\test",
                    @"-r",
                    @"-o:" + fileOutput,
                    @"..\..\test_files\test"
                });

                // assert the file output
                string actual = ReadContent(fileOutput, false);
                string expected = ReadContent(@"..\..\test_files\ExtendedStatistics_SampleReport.xml", false);

                Assert.AreEqual(expected, actual, "The file output is wrong");

                // assert console output is empty
                string consoleOutput = newConsole.ToString();
                Assert.AreEqual("", consoleOutput, "There's no console output");
            }
            finally
            {
                // deletes the output file
                System.IO.File.Delete(fileOutput);
            }
        }

        /// <summary>
        /// Tests CommandLine given a path that is relative to the base path.
        /// </summary>
        [Test]
        public void TestCommandLineRelativePath()
        {
            // run the command line
            CommandLine.Main(new string[]
                {
                    @"-c:..\..\test_files\FileStatistics_CommandLine.xml",
                    @"-b:..\..\test_files",
                    @"test"
                });

            // assert the xml output (1 sub-dir is reported)
            string actual = newConsole.ToString();

            AssertXmlChildCount(actual, "/report/files/file", 5);
            AssertXml(actual, "/report/directories/directory[@name=\"test\"]/line", "30");
            AssertXml(actual, "/report/overall/line", "30");
        }

        /// <summary>
        /// Tests CommandLine given a path that is relative to the base path (with backslash).
        /// </summary>
        [Test]
        public void TestCommandLineRelativePath2()
        {
            // run the command line
            CommandLine.Main(new string[]
                {
                    @"-c:..\..\test_files\FileStatistics_CommandLine.xml",
                    @"-b:..\..\test_files\",
                    @"test"
                });

            // assert the xml output (1 sub-dir is reported)
            string actual = newConsole.ToString();

            AssertXmlChildCount(actual, "/report/files/file", 5);
            AssertXml(actual, "/report/directories/directory[@name=\"test\"]/line", "30");
            AssertXml(actual, "/report/overall/line", "30");
        }

        /// <summary>
        /// Tests CommandLine given a non-existing path that is relative to the base path.
        /// </summary>
        [Test]
        public void TestCommandLineRelativePathNotExist()
        {
            // run the command line
            CommandLine.Main(new string[]
                {
                    @"-c:..\..\test_files\FileStatistics_CommandLine.xml",
                    @"-b:..\..\test_files",
                    @"testx"
                });

            // assert an error is reported
            string actual = newConsole.ToString();
            Assert.IsTrue(actual.IndexOf("Error") == 0,
                "An error message should be printed");
        }

        /// <summary>
        /// Tests CommandLine given a path without a base path.
        /// </summary>
        [Test]
        public void TestCommandLineAbsolutePath()
        {
            // run the command line
            CommandLine.Main(new string[]
                {
                    @"-c:..\..\test_files\FileStatistics_CommandLine.xml",
                    @"..\..\test_files\test"
                });

            // assert the xml output
            string actual = newConsole.ToString();

            AssertXmlChildCount(actual, "/report/files/file", 5);
            AssertXml(actual, "/report/overall/line", "30");
        }
    }
}
