/*
 * TestExtendedFileStatistics.cs
 *
 * File Statistics
 * Version: 1.0
 *
 * Copyright (c) 2005, TopCoder, Inc. All rights reserved
 *
 * Author: TCSDEVELOPER
 */

using System;
using System.Collections;
using System.IO;
using NUnit.Framework;
using TopCoder.File.Statistics.Matcher;
using TopCoder.File.Statistics.Processor.LineCounter;
using TopCoder.File.Statistics.Reporting;
using TopCoder.File.Statistics.Tests;

namespace TopCoder.File.Statistics.Tests
{
    /// <summary>
    /// Unit tests for the <see cref="ExtendedFileStatistics"/> class.
    /// </summary>
    [TestFixture]
    public class TestExtendedFileStatistics : TestHelper
    {
        /// <summary>
        /// The <see cref="ExtendedFileStatistics"/> instance to test.
        /// Initialized in SetUp().
        /// </summary>
        private ExtendedFileStatistics statistics;

        /// <summary>
        /// Initializes the <see cref="ExtendedFileStatistics"/> instance to test.
        /// </summary>
        [SetUp]
        public void SetUp()
        {
            // load test_files/FileStatistics.xml
            base.LoadConfigFile(ValidConfig);

            // initialize ExtendedFileStatistics
            statistics = new ExtendedFileStatistics(ValidNamespace);
        }

        /// <summary>
        /// Tests constructor(string) loads processors correctly
        /// and AllFileProcessors getter is correct.
        /// </summary>
        [Test]
        public void TestConstructorProcessor()
        {
            // 2 processors
            IList processors = statistics.AllFileProcessors;
            Assert.AreEqual(2, processors.Count, "2 processors are registered");

            // 1st processor
            IFileProcessor processor = processors[0] as IFileProcessor;
            Assert.AreEqual("Text File", processor.Alias,
                "The 1st processor alias is Text File");
            Assert.IsTrue(processor is TextLineCounter,
                "The 1st processor is TextLineCounter");
            Assert.IsTrue(processor.FileMatcher is ExtensionMatcher,
                "The 1st matcher is ExtensionMatcher");
            // assert the matcher can match only txt and sql
            Assert.IsTrue(processor.FileMatcher.Match(new FileInfo("test.txt")),
                "txt should match");
            Assert.IsTrue(processor.FileMatcher.Match(new FileInfo("test.sql")),
                "sql should match");
            Assert.IsFalse(processor.FileMatcher.Match(new FileInfo("test.bin")),
                "bin should not match");

            // 2nd processor
            processor = processors[1] as IFileProcessor;
            Assert.AreEqual("Source File", processor.Alias,
                "The 2nd processor alias is Source File");
            Assert.IsTrue(processor is CStyleLineCounter,
                "The 2nd processor is CStyleLineCounter");
            Assert.IsTrue(processor.FileMatcher is RegexMatcher,
                "The 2nd matcher is RegexMatcher");
            // assert the matcher can match only .c
            Assert.IsFalse(processor.FileMatcher.Match(new FileInfo("test.txt")),
                "txt should not match");
            Assert.IsFalse(processor.FileMatcher.Match(new FileInfo("test.sql")),
                "sql should not match");
            Assert.IsTrue(processor.FileMatcher.Match(new FileInfo("test.c")),
                "c should match");
        }

        /// <summary>
        /// Tests constructor(string) loads reporting instance correctly
        /// and Reporting getter/setter is correct.
        /// </summary>
        [Test]
        public void TestConstructorReporting()
        {
            // XmlStatsReporting
            IStatsReporting report = statistics.Reporting;
            Assert.IsTrue(report is XmlStatsReporting,
                "The XmlStatsReporting instance is used");
            // make sure stylesheet is loaded
            Assert.IsFalse(report.GenerateReport().StartsWith("<?xml"),
                "The report should not be in xml format");

            // set to another instance
            report = new BasicStatsReporting();
            statistics.Reporting = report;
            Assert.IsTrue(statistics.Reporting is BasicStatsReporting,
                "The BasicStatsReporting instance is now used");
            Assert.IsFalse(report.GenerateReport().StartsWith("<?xml"),
                "The report should not be in xml format");
        }

        /// <summary>
        /// Tests constructor(string) loads matchall with true given "on".
        /// </summary>
        [Test]
        public void TestConstructorMatchAllOn()
        {
            Assert.IsTrue(statistics.IsMatchAll, "The MatchAll property is on");
        }

        /// <summary>
        /// Tests constructor(string) loads matchall with true given "TruE".
        /// </summary>
        [Test]
        public void TestConstructorMatchAllTrue()
        {
            base.LoadConfigFile(@"..\..\test_files\FileStatistics_MatchAll_True.xml");

            statistics = new ExtendedFileStatistics(ValidNamespace);
            Assert.IsTrue(statistics.IsMatchAll, "The MatchAll property is TruE");
        }

        /// <summary>
        /// Tests constructor(string) loads matchall with true given "yEs".
        /// </summary>
        [Test]
        public void TestConstructorMatchAllYes()
        {
            base.LoadConfigFile(@"..\..\test_files\FileStatistics_MatchAll_Yes.xml");

            statistics = new ExtendedFileStatistics(ValidNamespace);
            Assert.IsTrue(statistics.IsMatchAll, "The MatchAll property is yEs");
        }

        /// <summary>
        /// Tests constructor(string) loads matchall with false
        /// if the property is not specified.
        /// </summary>
        [Test]
        public void TestConstructorMatchAllDefault()
        {
            base.LoadConfigFile(@"..\..\test_files\FileStatistics_MatchAll_Default.xml");

            statistics = new ExtendedFileStatistics(ValidNamespace);
            Assert.IsFalse(statistics.IsMatchAll,
                "The MatchAll property is missing and should default to false");
        }

        /// <summary>
        /// Tests constructor(string) loads AnyFileMatcher
        /// if no matcher is specified.
        /// </summary>
        [Test]
        public void TestConstructorAnyFileMatcher()
        {
            // load test_files/FileStatistics_NoMatcher.xml
            base.LoadConfigFile(@"..\..\test_files\FileStatistics_NoMatcher.xml");

            // initialize ExtendedFileStatistics
            statistics = new ExtendedFileStatistics(ValidNamespace);

            // 1 processor
            IList processors = statistics.AllFileProcessors;
            Assert.AreEqual(1, processors.Count, "1 processor is registered");

            // assert processor
            IFileProcessor processor = processors[0] as IFileProcessor;
            Assert.AreEqual("Text File", processor.Alias,
                "The processor alias is Text File");
            Assert.IsTrue(processor is TextLineCounter,
                "The processor is TextLineCounter");
            Assert.IsTrue(processor.FileMatcher is AnyFileMatcher,
                "The matcher is AnyFileMatcher");
        }

        /// <summary>
        /// Tests constructor(string) throws ConfigurationException
        /// if no processors.names is specified.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ConfigurationException))]
        public void TestConstructorNoProcessorsNames()
        {
            // load test_files/FileStatistics_Invalid_NoProcessorsNames.xml");
            base.LoadConfigFile(@"..\..\test_files\FileStatistics_Invalid_NoProcessorsNames.xml");

            // initialize ExtendedFileStatistics
            new ExtendedFileStatistics(ValidNamespace);
        }

        /// <summary>
        /// Tests constructor(string) throws ConfigurationException
        /// if the processor aliasname is missing.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ConfigurationException))]
        public void TestConstructorNoProcessorsAlias()
        {
            // load test_files/FileStatistics_Invalid_NoProcessorAlias.xml
            base.LoadConfigFile(@"..\..\test_files\FileStatistics_Invalid_NoProcessorAlias.xml");

            // initialize ExtendedFileStatistics
            new ExtendedFileStatistics(ValidNamespace);
        }

        /// <summary>
        /// Tests constructor(string) throws ConfigurationException
        /// if the processor.classname is not instance of IFileProcessor.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ConfigurationException))]
        public void TestConstructorInvalidProcessorsClassname()
        {
            // load test_files/FileStatistics_Invalid_InvalidProcessor.xml
            base.LoadConfigFile(@"..\..\test_files\FileStatistics_Invalid_InvalidProcessor.xml");

            // initialize ExtendedFileStatistics
            new ExtendedFileStatistics(ValidNamespace);
        }

        /// <summary>
        /// Tests constructor(string) throws ConfigurationException
        /// if no reporting.classname is specified.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ConfigurationException))]
        public void TestConstructorNoReporting()
        {
            // load test_files/FileStatistics_Invalid_NoReporting.xml
            base.LoadConfigFile(@"..\..\test_files\FileStatistics_Invalid_NoReporting.xml");

            // initialize ExtendedFileStatistics
            new ExtendedFileStatistics(ValidNamespace);
        }

        /// <summary>
        /// Tests constructor(string) throws ConfigurationException
        /// if reporting.classname is not instance of IStatsReporting.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ConfigurationException))]
        public void TestConstructorInvalidReporting()
        {
            // load test_files/FileStatistics_Invalid_InvalidReporting.xml
            base.LoadConfigFile(@"..\..\test_files\FileStatistics_Invalid_InvalidReporting.xml");

            // initialize ExtendedFileStatistics
            new ExtendedFileStatistics(ValidNamespace);
        }

        /// <summary>
        /// Tests constructor(string) throws ArgumentNullException
        /// given null namespace.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestConstructorNullNamespace()
        {
            new ExtendedFileStatistics(null);
        }

        /// <summary>
        /// Tests constructor(string) throws ArgumentException
        /// given empty namespace.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestConstructorEmptyNamespace()
        {
            new ExtendedFileStatistics(" ");
        }

        /// <summary>
        /// Tests constructor() accuracy.
        /// </summary>
        [Test]
        public void TestConstructorDefault()
        {
            // use default constructor
            statistics = new ExtendedFileStatistics();

            Assert.AreEqual(0, statistics.AllFileProcessors.Count, "No processor is registered");
            Assert.IsFalse(statistics.IsMatchAll, "MatchAll default to false");
            Assert.IsTrue(statistics.Reporting is XmlStatsReporting,
                "The default reporting is loaded");

            // the default reporting doesn't use stylesheet
            Assert.IsTrue(statistics.Reporting.GenerateReport().StartsWith("<?xml"),
                "The default reporting is xml");
        }

        /// <summary>
        /// Tests ProcessDirectory(DirectoryInfo) of test_files/test/aaa non-recursively.
        /// </summary>
        [Test]
        public void TestProcessDirectorySimple()
        {
            SetXmlReporting();

            // process 1 directory containing 1 text file
            statistics.ProcessDirectory(new DirectoryInfo(@"..\..\test_files\test\aaa"));

            string report = statistics.Reporting.GenerateReport();

            // assert the report
            AssertXml(report, "/report/overall/line", "3");
            AssertXml(report, "/report/files/file[@name=\"aaa\\sample.txt\"]/line", "3");
            AssertXml(report, "/report/directories/directory[@name=\"aaa\"]/line", "3");
            AssertXml(report, "/report/aliases/alias[@name=\"Text File\"]/line", "3");
        }

        /// <summary>
        /// Tests ProcessDirectory(DirectoryInfo) of test_files/test non-recursively.
        /// </summary>
        [Test]
        public void TestProcessDirectorySimple2()
        {
            SetXmlReporting();

            // process 1 directory containing 1 text file and 4 c files
            statistics.ProcessDirectory(new DirectoryInfo(@"..\..\test_files\test"));

            string report = statistics.Reporting.GenerateReport();

            // assert the report
            AssertXml(report, "/report/overall/line", "30");

            // the text file and the 4 c files
            AssertXml(report, "/report/files/file[@name=\"test.txt\"]/line", "4");
            AssertXml(report, "/report/files/file[@name=\"comment.c\"]/line", "0");
            AssertXml(report, "/report/files/file[@name=\"literal.c\"]/line", "0");
            AssertXml(report, "/report/files/file[@name=\"sample.c\"]/line", "22");
            AssertXml(report, "/report/files/file[@name=\"simple.c\"]/line", "4");

            // the directories (null because it's non recursive)
            AssertXml(report, "/report/directories/directory[@name=\"aaa\"]/line", null);

            // the aliases
            AssertXml(report, "/report/aliases/alias[@name=\"Text File\"]/line", "4");
            AssertXml(report, "/report/aliases/alias[@name=\"Source File\"]/line", "26");
        }

        /// <summary>
        /// Tests ProcessDirectory(DirectoryInfo, bool) of test_files/test non-recursively.
        /// </summary>
        [Test]
        public void TestProcessDirectoryNonRecursive()
        {
            SetXmlReporting();

            // process 1 directory containing 1 text file and 4 c files (non-recursive)
            statistics.ProcessDirectory(new DirectoryInfo(@"..\..\test_files\test"), false);

            string report = statistics.Reporting.GenerateReport();

            // assert the report
            AssertXml(report, "/report/overall/line", "30");

            // the text file and the 4 c files
            AssertXml(report, "/report/files/file[@name=\"test.txt\"]/line", "4");
            AssertXml(report, "/report/files/file[@name=\"comment.c\"]/line", "0");
            AssertXml(report, "/report/files/file[@name=\"literal.c\"]/line", "0");
            AssertXml(report, "/report/files/file[@name=\"sample.c\"]/line", "22");
            AssertXml(report, "/report/files/file[@name=\"simple.c\"]/line", "4");

            // the directories (null because it's non recursive)
            AssertXml(report, "/report/directories/directory[@name=\"aaa\"]/line", null);

            // the aliases
            AssertXml(report, "/report/aliases/alias[@name=\"Text File\"]/line", "4");
            AssertXml(report, "/report/aliases/alias[@name=\"Source File\"]/line", "26");
        }

        /// <summary>
        /// Tests ProcessDirectory(DirectoryInfo, bool) of test_files/test recursively.
        /// </summary>
        [Test]
        public void TestProcessDirectoryRecursive()
        {
            SetXmlReporting();

            // process 1 directory containing 1 text file and 4 c files and 1 sub folder (1 text file)
            statistics.ProcessDirectory(new DirectoryInfo(@"..\..\test_files\test"), true);

            string report = statistics.Reporting.GenerateReport();

            // assert the report
            AssertXml(report, "/report/overall/line", "33");

            // subfolder aaa contains 1 text file
            AssertXml(report, "/report/files/file[@name=\"aaa\\sample.txt\"]/line", "3");

            // the text file and the 4 c files
            AssertXml(report, "/report/files/file[@name=\"test.txt\"]/line", "4");
            AssertXml(report, "/report/files/file[@name=\"comment.c\"]/line", "0");
            AssertXml(report, "/report/files/file[@name=\"literal.c\"]/line", "0");
            AssertXml(report, "/report/files/file[@name=\"sample.c\"]/line", "22");
            AssertXml(report, "/report/files/file[@name=\"simple.c\"]/line", "4");

            // the directories (aaa)
            AssertXml(report, "/report/directories/directory[@name=\"aaa\"]/line", "3");

            // the aliases
            AssertXml(report, "/report/aliases/alias[@name=\"Text File\"]/line", "7");
            AssertXml(report, "/report/aliases/alias[@name=\"Source File\"]/line", "26");
        }

        /// <summary>
        /// Tests ProcessDirectory(DirectoryInfo) throws ArgumentNullException
        /// given null argument.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestProcessDirectoryNull()
        {
            statistics.ProcessDirectory(null);
        }

        /// <summary>
        /// Tests ProcessDirectory(DirectoryInfo, bool) throws ArgumentNullException
        /// given null argument.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestProcessDirectoryNull2()
        {
            statistics.ProcessDirectory(null, true);
        }

        /// <summary>
        /// Tests ProcessDirectory(DirectoryInfo) throws IOException
        /// given directory that doesn't exist.
        /// </summary>
        [Test]
        public void TestProcessDirectoryNotExist()
        {
            try
            {
                // this throws DirectoryNotFoundException, which is a subclass of IOException
                statistics.ProcessDirectory(new DirectoryInfo("notexists"));
            }
            catch (IOException)
            {
                // ok
            }
        }

        /// <summary>
        /// Tests ProcessDirectory(DirectoryInfo, bool) throws IOException
        /// given directory that doesn't exist.
        /// </summary>
        [Test]
        public void TestProcessDirectoryNotExist2()
        {
            try
            {
                // this throws DirectoryNotFoundException, which is a subclass of IOException
                statistics.ProcessDirectory(new DirectoryInfo("notexists"), true);
            }
            catch (IOException)
            {
                // ok
            }
        }

        /// <summary>
        /// Tests ProcessDirectory(DirectoryInfo) throws IOException
        /// given a file.
        /// </summary>
        [Test]
        [ExpectedException(typeof(IOException))]
        public void TestProcessDirectoryFile()
        {
            statistics.ProcessDirectory(new DirectoryInfo(SampleText));
        }

        /// <summary>
        /// Tests ProcessDirectory(DirectoryInfo, bool) throws IOException
        /// given a file.
        /// </summary>
        [Test]
        [ExpectedException(typeof(IOException))]
        public void TestProcessDirectoryFile2()
        {
            statistics.ProcessDirectory(new DirectoryInfo(SampleText), true);
        }

        /// <summary>
        /// Tests PrintReport() by recursively processing test_files\test directory.
        /// </summary>
        [Test]
        public void TestPrintReport()
        {
            SetXmlReporting();

            // process test_files/test recursively
            statistics.ProcessDirectory(new DirectoryInfo(@"..\..\test_files\test"), true);

            // run PrintReport, but first we'll redirect the console output to a string
            TextWriter oldOutput = Console.Out;
            try
            {
                // redirect console output to a StringWriter
                TextWriter writer = new StringWriter();
                Console.SetOut(writer);
                statistics.PrintReport();

                // assert the output
                string actual = writer.ToString();
                string expected = ReadContent(@"..\..\test_files\ExtendedStatistics_SampleReport.xml", false);

                Assert.AreEqual(expected, actual, "The generated output is wrong");
            }
            finally
            {
                // restore the standard output
                Console.SetOut(oldOutput);
            }
        }

        /// <summary>
        /// Tests PrintReport() to a file
        /// by recursively processing test_files\test directory.
        /// </summary>
        [Test]
        public void TestPrintReportFile()
        {
            SetXmlReporting();

            // process test_files/test recursively
            statistics.ProcessDirectory(new DirectoryInfo(@"..\..\test_files\test"), true);

            // run PrintReport and save to a file
            string fileOutput = @"..\..\test_files\TempOutput.xml";
            try
            {
                statistics.PrintReport(new FileInfo(fileOutput));

                // assert the output
                string actual = ReadContent(fileOutput, false);
                string expected = ReadContent(@"..\..\test_files\ExtendedStatistics_SampleReport.xml", false);

                Assert.AreEqual(expected, actual, "The generated output is wrong");
            }
            finally
            {
                // deletes the output file
                System.IO.File.Delete(fileOutput);
            }
        }

        /// <summary>
        /// Sets to plain xml reporting (no stylesheet)
        /// and set BaseDirectory to test_files/test
        /// </summary>
        private void SetXmlReporting()
        {
            statistics.Reporting = new XmlStatsReporting();
            statistics.Reporting.Reset(new DirectoryInfo(@"..\..\test_files\test"));
        }
    }
}
